/*

Copyright or © or Copr. Leo Ducas and Tancrede Lepoint.

Leo.Ducas@ens.fr and Tancrede.Lepoint@ens.fr

This software is a computer program whose purpose is to provide to the 
research community a proof-of-concept implementation of the BLISS 
digital signature scheme of Ducas, Durmus, Lepoint and Lyubashevsky 
appeared at Crypto 2013.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

*/

#ifndef _PARAMETERS_H
#define _PARAMETERS_H

#define CLASS 1
// 0 = Fun, 1 = speed, 2 = size, 3 = security 160, 4 = security 192

#if CLASS == 0

#define N 256
#define log2N 8

#define alpha_rejection 0.5
#define kappa 12
#define density 0.55
#define density2 0.15

#define C 1.5

#define sigma 100
#define dropped_bits 5
#define B_VERIFY 2428*2428
#define BINFTY 542
#define Q 7681

#define oneQTwo 3841// 1/(q+2) mod 2q
#define modp 480

#elif CLASS == 1

#define N 512
#define log2N 9
#define alpha_rejection 1
#define Q 12289
#define kappa 23
#define density 0.3
#define density2 0
#define C 1.62
#define sigma 215
#define B_VERIFY 12872*12872
#define BINFTY 2100

#define dropped_bits 10
#define oneQTwo 6145 // 1/(q+2) mod 2q

#define modp 24

#elif CLASS == 2

#define N 512
#define log2N 9
#define alpha_rejection 0.5
#define Q 12289
#define kappa 23
#define density 0.3
#define density2 0
#define C 1.62
#define sigma 107
#define B_VERIFY 11073*11073
#define BINFTY 1563

#define dropped_bits 10
#define oneQTwo 6145 // 1/(q+2) mod 2q

#define modp 24

#elif CLASS == 3

#define N 512
#define log2N 9
#define alpha_rejection 0.7
#define Q 12289
#define kappa 30
#define density 0.42
#define density2 0.03
#define C 1.75
#define sigma 250
#define B_VERIFY 10206*10206
#define BINFTY 1760

#define dropped_bits 9
#define oneQTwo 6145 // 1/(q+2) mod 2q

#define modp 48

#elif CLASS == 4

#define N 512
#define log2N 9
#define alpha_rejection 0.55
#define Q 12289
#define kappa 39
#define density 0.45
#define density2 0.06
#define C 1.88
#define sigma 271
//#define B_VERIFY 8950*8950
#define B_VERIFY 9901*9901
#define BINFTY 1613

#define dropped_bits 8
#define oneQTwo 6145 // 1/(q+2) mod 2q

#define modp 96

#endif

// FFT

#if (Q==7681)

#define normalization_factor 7651 // 1/N mod Q
#define oneoverQ 559167 // floor(2^32/Q)
#define oneover2Q 279583 // floor(2^32/2Q)

// W^i; W=root of g^(2n)-1
const long W[2*N+1] = {1, 62, 3844, 217, 5773, 4600, 1003, 738, 7351, 2583, 6526, 5200, 7479,
2838, 6974, 2252, 1366, 201, 4781, 4544, 5212, 542, 2880, 1897, 2399,
2799, 4556, 5956, 584, 5484, 2044, 3832, 7154, 5731, 1996, 856, 6986,
2996, 1408, 2805, 4928, 5977, 1886, 1717, 6601, 2169, 3901, 3751, 2132,
1607, 7462, 1784, 3074, 6244, 3078, 6492, 3092, 7360, 3141, 2717, 7153,
5669, 5833, 639, 1213, 6077, 405, 2067, 5258, 3394, 3041, 4198, 6803,
7012, 4608, 1499, 766, 1406, 2681, 4921, 5543, 5702, 198, 4595, 693,
4561, 6266, 4442, 6569, 185, 3789, 4488, 1740, 346, 6090, 1211, 5953,
398, 1633, 1393, 1875, 1035, 2722, 7463, 1846, 6918, 6461, 1170, 3411,
4095, 417, 2811, 5300, 5998, 3188, 5631, 3477, 506, 648, 1771, 2268,
2358, 257, 572, 4740, 2002, 1228, 7007, 4298, 5322, 7362, 3265, 2724,
7587, 1853, 7352, 2645, 2689, 5417, 5571, 7438, 296, 2990, 1036, 2784,
3626, 2063, 5010, 3380, 2173, 4149, 3765, 3000, 1656, 2819, 5796, 6026,
4924, 5729, 1872, 849, 6552, 6812, 7570, 799, 3452, 6637, 4401, 4027,
3882, 2573, 5906, 5165, 5309, 6556, 7060, 7584, 1667, 3501, 1994, 732,
6979, 2562, 5224, 1286, 2922, 4501, 2546, 4232, 1230, 7131, 4305, 5756,
3546, 4784, 4730, 1382, 1193, 4837, 335, 5408, 5013, 3566, 6024, 4800,
5722, 1438, 4665, 5033, 4806, 6094, 1459, 5967, 1266, 1682, 4431, 5887,
3987, 1402, 2433, 4907, 4675, 5653, 4841, 583, 5422, 5881, 3615, 1381,
1131, 993, 118, 7316, 413, 2563, 5286, 5130, 3139, 2593, 7146, 5235,
1968, 6801, 6888, 4601, 1065, 4582, 7568, 675, 3445, 6203, 536, 2508,
1876, 1097, 6566, 7680, 7619, 3837, 7464, 1908, 3081, 6678, 6943, 330,
5098, 1155, 2481, 202, 4843, 707, 5429, 6315, 7480, 2900, 3137, 2469,
7139, 4801, 5784, 5282, 4882, 3125, 1725, 7097, 2197, 5637, 3849, 527,
1950, 5685, 6825, 695, 4685, 6273, 4876, 2753, 1704, 5795, 5964, 1080,
5512, 3780, 3930, 5549, 6074, 219, 5897, 4607, 1437, 4603, 1189, 4589,
321, 4540, 4964, 528, 2012, 1848, 7042, 6468, 1604, 7276, 5614, 2423,
4287, 4640, 3483, 878, 669, 3073, 6182, 6915, 6275, 5000, 2760, 2138,
1979, 7483, 3086, 6988, 3120, 1415, 3239, 1112, 7496, 3892, 3193, 5941,
7335, 1591, 6470, 1728, 7283, 6048, 6288, 5806, 6646, 4959, 218, 5835,
763, 1220, 6511, 4270, 3586, 7264, 4870, 2381, 1683, 4493, 2050, 4204,
7175, 7033, 5910, 5413, 5323, 7424, 7109, 2941, 5679, 6453, 674, 3383,
2359, 319, 4416, 4957, 94, 5828, 329, 5036, 4992, 2264, 2110, 243, 7385,
4691, 6645, 4897, 4055, 5618, 2671, 4301, 5508, 3532, 3916, 4681, 6025,
4862, 1885, 1655, 2757, 1952, 5809, 6832, 1129, 869, 111, 6882, 4229,
1044, 3280, 3654, 3799, 5108, 1775, 2516, 2372, 1125, 621, 97, 6014,
4180, 5687, 6949, 702, 5119, 2457, 6395, 4759, 3180, 5135, 3449, 6451,
550, 3376, 1925, 4135, 2897, 2951, 6299, 6488, 2844, 7346, 2273, 2668,
4115, 1657, 2881, 1959, 6243, 3016, 2648, 2875, 1587, 6222, 1714, 6415,
5999, 3250, 1794, 3694, 6279, 5248, 2774, 3006, 2028, 2840, 7098, 2259,
1800, 4066, 6300, 6550, 6688, 7563, 365, 7268, 5118, 2395, 2551, 4542,
5088, 535, 2446, 5713, 880, 793, 3080, 6616, 3099, 113, 7006, 4236,
1478, 7145, 5173, 5805, 6584, 1115, 1}; 

//
//for i in range(513):                                              
//    v[i] = floor((2^(64)*((62**i)%7681)/7681))
//
const unsigned long W2[2*N+1] = {2401607092007492U, 148899639704464548U, 9231777661676802032U,
521148738965625921U, 13864477742159255497U, 11047392623234466532U,
2408811913283515202U, 1772386033901529630U, 17654213733347079016U,
6203351118655353707U, 15672887882440897519U, 12488356878438962166U,
17961619441124038085U, 6815760927117264351U, 16748807859660254259U,
5408419171200873615U, 3280595287682235061U, 482723025493506037U,
11482083506887822715U, 10912902626082046939U, 12517176163543052079U,
1301671043868061056U, 6916628424981579046U, 4555848653538213698U,
5761455413725975045U, 6722098250528972135U, 10941721911186136852U,
14303971839996626666U, 1402538541732375751U, 13170413292569090100U,
4908884896063315128U, 9202958376572712119U, 17181097136221602950U,
13763610244294940803U, 4793607755646955477U, 2055775670758413772U,
16777627144764344172U, 7195214847654448202U, 3381462785546549755U,
6736507893081017091U, 11835119749412924145U, 14354405588928784013U,
4529430975526131278U, 4123559376976865007U, 15853008414341459473U,
5209085782564251719U, 9368669265921229117U, 9008428202120105209U,
5120226320159974488U, 3859382596856040808U, 17920792120559910709U,
4284467052141367020U, 7382540200831032634U, 14995634682494784571U,
7392146629199062605U, 15591233241312642766U, 7425769128487167503U,
17675828197175146451U, 7543447875995534647U, 6525166468984357732U,
17178695529129595457U, 13614710604590476254U, 14008574167679705061U,
1534626931792787850U, 2913149402605088674U, 14594566298129533286U,
972650872263034553U, 4964121859179487461U, 12627650089775396744U,
8151054470273430306U, 7303287166794785374U, 10081946572247454456U,
16338133046926973003U, 16840068929156538983U, 11066605479970526473U,
3600009030919231593U, 1839631032477739426U, 3376659571362534770U,
6438708613672087994U, 11818308499768871696U, 13312108110997532171U,
13693963638626723514U, 475518204217483559U, 11035384587774429068U,
1664313714761192457U, 10953729946646174315U, 15048470038518949410U,
10667938702697282681U, 15776156987397219706U, 444297312021386154U,
9099689271616389932U, 10778412628929627347U, 4178796340093037340U,
830956053834592482U, 14625787190325630691U, 2908346188421073689U,
14296767018720604188U, 955839622618982104U, 3921824381248235618U,
3345438679166437365U, 4503013297514048858U, 2485663340227754969U,
6537174504444395195U, 17923193727651918202U, 4433366691845831569U,
16614317862507834667U, 15516783421460410492U, 2809880297648766487U,
8191881790837557682U, 9834581041770682706U, 1001470157367124466U,
6750917535633062048U, 12728517587639711439U, 14404839337860941360U,
7656323409319886805U, 13523449535094191531U, 8350387858910052202U,
1215213188555791318U, 1556241395620855285U, 4253246159945269614U,
5446844884672993498U, 5662989522953667844U, 617213022645925630U,
1373719256628285838U, 11383617616115515513U, 4808017398199000434U,
2949173508985201065U, 16828060893696501519U, 10322107281448203729U,
12781352943663876279U, 17680631411359161436U, 7841247155404463745U,
6541977718628410181U, 18220993007060847299U, 4450177941489884018U,
17656615340439086509U, 6352250758359818255U, 6457921470408147935U,
13009505617404588087U, 13379353109573741967U, 17863153550351730883U,
710875699234217846U, 7180805205102403245U, 2488064947319762462U,
6686074144148859744U, 8708227315619168618U, 4954515430811457490U,
12032051530957538549U, 8117431970985325408U, 5218692210932281690U,
9964267824739087313U, 9042050701408210107U, 7204821276022478173U,
3977061344364407951U, 6770130392369121990U, 13919714705275427830U,
14472084336437151157U, 11825513321044894174U, 13758807030110925818U,
4495808476238026380U, 2038964421114361323U, 15735329666833092330U,
16359747510755040438U, 18180165686496719923U, 1918884066513986686U,
8290347681609864884U, 15939466269653729211U, 10569472811924975480U,
9671271759514173201U, 9323038731173086756U, 6179335047735278779U,
14183891485396252030U, 12404300630218699921U, 12750132051467778873U,
15744936095201122301U, 16955346069572898634U, 18213788185784824821U,
4003479022376490371U, 8408026429118232028U, 4788804541462940492U,
1757976391349484674U, 16760815895120291723U, 6152917369723196359U,
12545995448647141992U, 3088466720321635643U, 7017495922845893740U,
10809633521125724752U, 6114491656251076476U, 10163601213375709209U,
2953976723169216050U, 17125860173105430617U, 10338918531092256178U,
13823650421595128121U, 8516098748258569200U, 11489288328163845193U,
11359601545195440586U, 3319021001154354945U, 2865117260764938820U,
11616573504040242307U, 804538375822510062U, 12987891153576520653U,
12039256352233561027U, 8564130890098719055U, 14467281122253136171U,
11527714041635965077U, 13741995780466873368U, 3453510998306774537U,
11203497084214953559U, 12087288494073710881U, 11542123684188010033U,
14635393618693660662U, 3503944747238931884U, 14330389518008709086U,
3040434578481485789U, 4039503128756602762U, 10641521024685200261U,
14138260950648109668U, 9575207475833873492U, 3367053142994504799U,
5843110054854229798U, 11784686000480766798U, 11227513155135028486U,
13576284891118356370U, 11626179932408272278U, 1400136934640368258U,
13021513652864625551U, 14123851308096064712U, 8681809637607086198U,
3316619394062347452U, 2716217621060474271U, 2384795842363440275U,
283389636856884141U, 17570157485126816771U, 991863728999094495U,
6155318976815203852U, 12694895088351606541U, 12320244381998437676U,
7538644661811519661U, 6227367189575428634U, 17161884279485543008U,
12572413126659224412U, 4726362757070745681U, 16333329832742958018U,
16542269649747609885U, 11049794230326474024U, 2557711552987979751U,
11004163695578331663U, 18175362472312704938U, 1621084787105057588U,
8273536431965812435U, 14897168791722477369U, 1287261401316016100U,
6023230586754791752U, 4505414904606056350U, 2634562979932219518U,
15768952166121197228U, 18444342466617544123U, 18297844434005087067U,
9214966412032749583U, 17925595334743925694U, 4582266331550296118U,
7399351450475085083U, 16037932160426036413U, 16674358039808021985U,
792530340362472599U, 12243392955054197908U, 2773856191268654096U,
5958387195270589449U, 485124632585513530U, 11630983146592287264U,
1697936214049297356U, 13038324902508678000U, 15166148786027316554U,
17964021048216045578U, 6964660566821728900U, 7533841447627504676U,
5929567910166499536U, 17145073029841490559U, 11530115648727972569U,
13890895420171337917U, 12685288659983576570U, 11724645823180579480U,
7505022162523414763U, 4142772233712924949U, 17044205531977175864U,
5276330781140461515U, 13537859177646236487U, 9243785697136839496U,
1265646937487948665U, 4683133829414610812U, 13653136318062596138U,
16390968402951137843U, 1669116928945207443U, 11251529226055103413U,
15065281288163001860U, 11710236180628534524U, 6611624324296627470U,
4092338484780767602U, 13917313098183420337U, 14323184696732686608U,
2593735659368092142U, 13237658291145299896U, 9078074807788322498U,
9438315871589446406U, 13326517753549577127U, 14587361476853510807U,
525951953149640906U, 14162277021568184595U, 11064203872878518981U,
3451109391214767044U, 11054597444510489010U, 2855510832396908849U,
11020974945222384112U, 770915876534405164U, 10903296197714016968U,
11921577604725193883U, 1268048544579956158U, 4832033469119075361U,
4438169906029846554U, 16912117141916763765U, 15533594671104462941U,
3852177775580018329U, 17474093201446517062U, 13482622214530064154U,
5819093983934154871U, 10295689603436121309U, 11143456906914766241U,
8364797501462097159U, 2108611026782578612U, 1606675144553012632U,
7380138593739025142U, 14846735042790320022U, 16607113041231812189U,
15070084502347016845U, 12008035460037463621U, 6628435573940679919U,
5134635962712019444U, 4752780435082828101U, 17971225869492068056U,
7411359485935122547U, 16782430358948359158U, 7493014127063377300U,
3398274035190602205U, 7778805371012268934U, 2670587086312331909U,
18002446761688165461U, 9347054802093161683U, 7668331444779924268U,
14267947733616514275U, 17615788019874959133U, 3820956883383920924U,
15538397885288477926U, 4149977054988947427U, 17490904451090569511U,
14524919692461315997U, 15101305394543114250U, 13943730776195502757U,
15961080733481796646U, 11909569569265156420U, 523550346057633413U,
14013377381863720046U, 1832426211201716948U, 2929960652249141123U,
15636863776060785128U, 10254862282871993933U, 8612163031938868909U,
17445273916342427149U, 11695826538076489567U, 5718226486069840176U,
4041904735848610255U, 10790420664389664810U, 4923294538615360084U,
10096356214799499413U, 17231530885153760297U, 16890502678088696330U,
14193497913764282001U, 12999899189036558117U, 12783754550755883771U,
17829531051063625985U, 17073024817081265777U, 7063126457594036102U,
13638726675510551181U, 15497570564724350550U, 1618683180013050096U,
8124636792261347886U, 5665391130045675336U, 766112662350390179U,
10605496918305087870U, 11904766355081141434U, 225751066648704316U,
13996566132219667597U, 790128733270465106U, 12094493315349733360U,
11988822603301403680U, 5437238456304963528U, 5067390964135809648U,
583590523357820732U, 17735868374475333769U, 11265938868607148370U,
15958679126389789153U, 11760669929560691871U, 9738516758090382997U,
13492228642898094125U, 6414692542752013066U, 10329312102724226207U,
13228051862777269925U, 8482476248970464302U, 9404693372301341508U,
11241922797687073442U, 14469682729345143664U, 11676613681340429625U,
4527029368434123785U, 3974659737272400458U, 6621230752664657441U,
4687937043598625797U, 13950935597471525235U, 16407779652595190292U,
2711414406876459285U, 2086996562954511177U, 266578387212831692U,
16527860007195564929U, 10156396392099686731U, 2507277804055822404U,
7877271261784576135U, 8775472314195378414U, 9123705342536464859U,
12267409025974272836U, 4262852588313299585U, 6042443443490851694U,
5696612022241772742U, 2701807978508429314U, 1491398004136652981U,
232955887924726794U, 14443265051333061244U, 10038717644591319587U,
13657939532246611123U, 16688767682360066941U, 1685928178589259892U,
12293826703986355256U, 5900748625062409623U, 15358277353387915972U,
11429248150863657875U, 7637110552583826863U, 12332252417458475139U,
8283142860333842406U, 15492767350540335565U, 1320883900604120998U,
8107825542617295437U, 4623093652114423494U, 9930645325450982415U,
6957455745545706422U, 7087142528514111029U, 15127723072555196670U,
15581626812944612795U, 6830170569669309308U, 17642205697887041553U,
5458852920133030962U, 6407487721475990588U, 9882613183610832560U,
3979462951456415444U, 6919030032073586538U, 4704748293242678247U,
14993233075402777078U, 7243246989494598056U, 6359455579635840734U,
6904620389521541582U, 3811350455015890953U, 14942799326470619731U,
4116354555700842529U, 15406309495228065826U, 14407240944952948853U,
7805223049024351354U, 4308483123061441947U, 8871536597875678123U,
15079690930715046816U, 12603634018855321817U, 6662058073228784817U,
7219230918574523129U, 4870459182591195245U, 6820564141301279337U,
17046607139069183357U, 5425230420844926064U, 4322892765613486903U,
9764934436102465417U, 15130124679647204163U, 15730526452649077344U,
16061948231346111340U, 18163354436852667474U, 876586588582734844U,
17454880344710457120U, 12291425096894347763U, 5751848985357945074U,
6126499691711113939U, 10908099411898031954U, 12219376884134122981U,
1284859794224008607U, 5874330947050327203U, 13720381316638805934U,
2113414240966593597U, 1904474423961941730U, 7396949843383077591U,
15889032520721571864U, 7442580378131219952U, 271381601396846677U,
16825659286604494027U, 10173207641743739180U, 3549575281987074246U,
17159482672393535515U, 12423513486954759863U, 13941329169103495265U,
15812181093777332097U, 2677791907588354387U, 2401607092007492U}; 

const long flip[N] = {0, 128, 64, 192, 32, 160, 96, 224, 16, 144, 80, 208, 48, 176, 112, 240,
8, 136, 72, 200, 40, 168, 104, 232, 24, 152, 88, 216, 56, 184, 120, 248,
4, 132, 68, 196, 36, 164, 100, 228, 20, 148, 84, 212, 52, 180, 116, 244,
12, 140, 76, 204, 44, 172, 108, 236, 28, 156, 92, 220, 60, 188, 124,
252, 2, 130, 66, 194, 34, 162, 98, 226, 18, 146, 82, 210, 50, 178, 114,
242, 10, 138, 74, 202, 42, 170, 106, 234, 26, 154, 90, 218, 58, 186,
122, 250, 6, 134, 70, 198, 38, 166, 102, 230, 22, 150, 86, 214, 54, 182,
118, 246, 14, 142, 78, 206, 46, 174, 110, 238, 30, 158, 94, 222, 62,
190, 126, 254, 1, 129, 65, 193, 33, 161, 97, 225, 17, 145, 81, 209, 49,
177, 113, 241, 9, 137, 73, 201, 41, 169, 105, 233, 25, 153, 89, 217, 57,
185, 121, 249, 5, 133, 69, 197, 37, 165, 101, 229, 21, 149, 85, 213, 53,
181, 117, 245, 13, 141, 77, 205, 45, 173, 109, 237, 29, 157, 93, 221,
61, 189, 125, 253, 3, 131, 67, 195, 35, 163, 99, 227, 19, 147, 83, 211,
51, 179, 115, 243, 11, 139, 75, 203, 43, 171, 107, 235, 27, 155, 91,
219, 59, 187, 123, 251, 7, 135, 71, 199, 39, 167, 103, 231, 23, 151, 87,
215, 55, 183, 119, 247, 15, 143, 79, 207, 47, 175, 111, 239, 31, 159,
95, 223, 63, 191, 127, 255};

#elif (Q==12289)

#define normalization_factor 12265 // 1/N mod Q
#define oneoverQ 349496 // floor(2^32/Q)
#define oneover2Q 174748 // floor(2^32/2Q)


// v = [F(49)**i for i in range(1025)]; 
const long W[2*N+1] = {1, 49, 2401, 7048, 1260, 295, 2166, 7822, 2319, 3030,
1002, 12231, 9447, 8210, 9042, 654, 7468, 9551, 1017, 677, 8595, 3329, 3364,
5079, 3091, 3991, 11224, 9260, 11336, 2459, 9890, 5339, 3542, 1512, 354, 5057,
2013, 325, 3636, 6118, 4846, 3963, 9852, 3477, 10616, 4046, 1630, 6136, 5728,
10314, 1537, 1579, 3637, 6167, 7247, 11011, 11112, 3772, 493, 11868, 3949,
9166, 6730, 10256, 10984, 9789, 390, 6821, 2426, 8273, 12129, 4449, 9088,
2908, 7313, 1956, 9821, 1958, 9919, 6760, 11726, 9280, 27, 1323, 3382, 5961,
9442, 7965, 9326, 2281, 1168, 8076, 2476, 10723, 9289, 468, 10643, 5369, 5012,
12097, 2881, 5990, 10863, 3860, 4805, 1954, 9723, 9445, 8112, 4240, 11136,
4948, 8961, 8974, 9611, 3957, 9558, 1360, 5195, 8775, 12149, 5429, 7952, 8689,
7935, 7856, 3985, 10930, 7143, 5915, 7188, 8120, 4632, 5766, 12176, 6752,
11334, 2361, 5088, 3532, 1022, 922, 8311, 1702, 9664, 6554, 1632, 6234, 10530,
12121, 4057, 2169, 7969, 9522, 11885, 4782, 827, 3656, 7098, 3710, 9744,
10474, 9377, 4780, 729, 11143, 5291, 1190, 9154, 6142, 6022, 142, 6958, 9139,
5407, 6874, 5023, 347, 4714, 9784, 145, 7105, 4053, 1973, 10654, 5908, 6845,
3602, 4452, 9235, 10111, 3879, 5736, 10706, 8456, 8807, 1428, 8527, 12286,
12142, 5086, 3434, 8509, 11404, 5791, 1112, 5332, 3199, 9283, 174, 8526,
12237, 9741, 10327, 2174, 8214, 9238, 10258, 11082, 2302, 2197, 9341, 3016,
316, 3195, 9087, 2859, 4912, 7197, 8561, 1663, 7753, 11227, 9407, 6250, 11314,
1381, 6224, 10040, 400, 7311, 1858, 5019, 151, 7399, 6170, 7394, 5925, 7678,
7552, 1378, 6077, 2837, 3834, 3531, 973, 10810, 1263, 442, 9369, 4388, 6099,
3915, 7500, 11119, 4115, 5011, 12048, 480, 11231, 9603, 3565, 2639, 6421,
7404, 6415, 7110, 4298, 1689, 9027, 12208, 8320, 2143, 6695, 8541, 683, 8889,
5446, 8785, 350, 4861, 4698, 9000, 10885, 4938, 8471, 9542, 576, 3646, 6608,
4278, 709, 10163, 6427, 7698, 8532, 242, 11858, 3459, 9734, 9984, 9945, 8034,
418, 8193, 8209, 8993, 10542, 420, 8291, 722, 10800, 773, 1010, 334, 4077,
3149, 6833, 3014, 218, 10682, 7280, 339, 4322, 2865, 5206, 9314, 1693, 9223,
9523, 11934, 7183, 7875, 4916, 7393, 5876, 5277, 504, 118, 5782, 671, 8301,
1212, 10232, 9808, 1321, 3284, 1159, 7635, 5445, 8736, 10238, 10102, 3438,
8705, 8719, 9405, 6152, 6512, 11863, 3704, 9450, 8357, 3956, 9509, 11248,
10436, 7515, 11854, 3263, 130, 6370, 4905, 6854, 4043, 1483, 11222, 9162,
6534, 652, 7370, 4749, 11499, 10446, 8005, 11286, 9, 441, 9320, 1987, 11340,
2655, 7205, 8953, 8582, 2692, 9018, 11767, 11289, 156, 7644, 5886, 5767,
12225, 9153, 6093, 3621, 5383, 5698, 8844, 3241, 11341, 2704, 9606, 3712,
9842, 2987, 11184, 7300, 1319, 3186, 8646, 5828, 2925, 8146, 5906, 6747,
11089, 2645, 6715, 9521, 11836, 2381, 6068, 2396, 6803, 1544, 1922, 8155,
6347, 3778, 787, 1696, 9370, 4437, 8500, 10963, 8760, 11414, 6281, 544, 2078,
3510, 12233, 9545, 723, 10849, 3174, 8058, 1594, 4372, 5315, 2366, 5333, 3248,
11684, 7222, 9786, 243, 11907, 5860, 4493, 11244, 10240, 10200, 8240, 10512,
11239, 9995, 10484, 9867, 4212, 9764, 11454, 8241, 10561, 1351, 4754, 11744,
10162, 6378, 5297, 1484, 11271, 11563, 1293, 1912, 7665, 6915, 7032, 476,
11035, 12288, 12240, 9888, 5241, 11029, 11994, 10123, 4467, 9970, 9259, 11287,
58, 2842, 4079, 3247, 11635, 4821, 2738, 11272, 11612, 3694, 8960, 8925, 7210,
9198, 8298, 1065, 3029, 953, 9830, 2399, 6950, 8747, 10777, 11935, 7232,
10276, 11964, 8653, 6171, 7443, 8326, 2437, 8812, 1673, 8243, 10659, 6153,
6561, 1975, 10752, 10710, 8652, 6122, 5042, 1278, 1177, 8517, 11796, 421,
8340, 3123, 5559, 2033, 1305, 2500, 11899, 5468, 9863, 4016, 160, 7840, 3201,
9381, 4976, 10333, 2468, 10331, 2370, 5529, 563, 3009, 12262, 10966, 8907,
6328, 2847, 4324, 2963, 10008, 11121, 4213, 9813, 1566, 3000, 11821, 1646,
6920, 7277, 192, 9408, 6299, 1426, 8429, 7484, 10335, 2566, 2844, 4177, 8049,
1153, 7341, 3328, 3315, 2678, 8332, 2731, 10929, 7094, 3514, 140, 6860, 4337,
3600, 4354, 4433, 8304, 1359, 5146, 6374, 5101, 4169, 7657, 6523, 113, 5537,
955, 9928, 7201, 8757, 11267, 11367, 3978, 10587, 2625, 5735, 10657, 6055,
1759, 168, 8232, 10120, 4320, 2767, 404, 7507, 11462, 8633, 5191, 8579, 2545,
1815, 2912, 7509, 11560, 1146, 6998, 11099, 3135, 6147, 6267, 12147, 5331,
3150, 6882, 5415, 7266, 11942, 7575, 2505, 12144, 5184, 8236, 10316, 1635,
6381, 5444, 8687, 7837, 3054, 2178, 8410, 6553, 1583, 3833, 3482, 10861, 3762,
3, 147, 7203, 8855, 3780, 885, 6498, 11177, 6957, 9090, 3006, 12115, 3763, 52,
2548, 1962, 10115, 4075, 3051, 2031, 1207, 9987, 10092, 2948, 9273, 11973,
9094, 3202, 9430, 7377, 5092, 3728, 10626, 4536, 1062, 2882, 6039, 975, 10908,
6065, 2249, 11889, 4978, 10431, 7270, 12138, 4890, 6119, 4895, 6364, 4611,
4737, 10911, 6212, 9452, 8455, 8758, 11316, 1479, 11026, 11847, 2920, 7901,
6190, 8374, 4789, 1170, 8174, 7278, 241, 11809, 1058, 2686, 8724, 9650, 5868,
4885, 5874, 5179, 7991, 10600, 3262, 81, 3969, 10146, 5594, 3748, 11606, 3400,
6843, 3504, 11939, 7428, 7591, 3289, 1404, 7351, 3818, 2747, 11713, 8643,
5681, 8011, 11580, 2126, 5862, 4591, 3757, 12047, 431, 8830, 2555, 2305, 2344,
4255, 11871, 4096, 4080, 3296, 1747, 11869, 3998, 11567, 1489, 11516, 11279,
11955, 8212, 9140, 5456, 9275, 12071, 1607, 5009, 11950, 7967, 9424, 7083,
2975, 10596, 3066, 2766, 355, 5106, 4414, 7373, 4896, 6413, 7012, 11785,
12171, 6507, 11618, 3988, 11077, 2057, 2481, 10968, 9005, 11130, 4654, 6844,
3553, 2051, 2187, 8851, 3584, 3570, 2884, 6137, 5777, 426, 8585, 2839, 3932,
8333, 2780, 1041, 1853, 4774, 435, 9026, 12159, 5919, 7384, 5435, 8246, 10806,
1067, 3127, 5755, 11637, 4919, 7540, 790, 1843, 4284, 1003, 12280, 11848,
2969, 10302, 949, 9634, 5084, 3336, 3707, 9597, 3271, 522, 1000, 12133, 4645,
6403, 6522, 64, 3136, 6196, 8668, 6906, 6591, 3445, 9048, 948, 9585, 2683,
8577, 2447, 9302, 1105, 4989, 10970, 9103, 3643, 6461, 9364, 4143, 6383, 5542,
1200, 9644, 5574, 2768, 453, 9908, 6221, 9893, 5486, 10745, 10367, 4134, 5942,
8511, 11502, 10593, 2919, 7852, 3789, 1326, 3529, 875, 6008, 11745, 10211,
8779, 56, 2744, 11566, 1440, 9115, 4231, 10695, 7917, 6974, 9923, 6956, 9041,
605, 5067, 2503, 12046, 382, 6429, 7796, 1045, 2049, 2089, 4049, 1777, 1050,
2294, 1805, 2422, 8077, 2525, 835, 4048, 1728, 10938, 7535, 545, 2127, 5911,
6992, 10805, 1018, 726, 10996, 10377, 4624, 5374, 5257, 11813, 1254, 1};


// for i in range(1025):                                              
//    v[i] = floor((2^(64)*((49**i)%12289)/12289))
const unsigned long W2[2*N+1] = {1501077717772768U, 73552808170865654U,
3604087600372417074U, 10579595754862472112U, 1891357924393688260U,
442817926742966695U, 3251334336695816486U, 11741429908418594901U,
3480999227515050060U, 4548265484851488436U, 1504079873208313997U,
18359681566078731045U, 14180681199799343650U, 12323848062914429063U,
13572744724101372423U, 981704827423390573U, 11210048396327034866U,
14336793282447711570U, 1526596038974905524U, 1016229614932164248U,
12901762984256944921U, 4997087722465546206U, 5049625442587593102U,
7623973728567891012U, 4639831225635627312U, 5990801171631118927U,
16848096304281553205U, 13899979666575835947U, 17016217008672103272U,
3691150108003237645U, 14845658628772680078U, 8014253935188810812U,
5316817276351145888U, 2269629509272425912U, 531381512091560035U,
7590950018776890106U, 3021669445876582911U, 487850258276149749U,
5457918581821786123U, 9183593477333797443U, 7274222620326835961U,
5948770995533481410U, 14788617675497314876U, 5219247224695915938U,
15935441051875709980U, 6073360446108621192U, 2446756679969612591U,
9210612876253707276U, 8598173167402417744U, 15482115581108333905U,
2307156452216745124U, 2370201716363201399U, 5459419659539558892U,
9257146285504663098U, 10878310220699253036U, 16528366750395953522U,
16679975599891003137U, 5662065151438882634U, 740031314861974851U,
17814790354527216093U, 5927755907484662652U, 13758878361105195712U,
10102253040610731741U, 15395053073477513334U, 16487837652016088774U,
14694049779277630463U, 585420309931379699U, 10238851112928053671U,
3641614543316736286U, 12418415959134113477U, 18206571638865908662U,
6678294766371046882U, 13641794299118919772U, 4365134003283210684U,
10977381350072255754U, 2936108015963535109U, 14742084266246359054U,
2939110171399080646U, 14889189882588090363U, 10147285372143914795U,
17601637318603482972U, 13930001220931291317U, 40529098379864748U,
1985925820613372673U, 5076644841507502934U, 8947924275643472795U,
14173175811210479807U, 11956084022060100791U, 13999050795948838666U,
3423958274239684859U, 1753258774358593562U, 12122703648732878090U,
3716668429205374709U, 16096056367677396206U, 13943510920391246233U,
702504371917655639U, 15975970150255574729U, 8059286266721993866U,
7523401521477115526U, 18158537151897180071U, 4324604904903345935U,
8991455529458883080U, 16306207248165583790U, 5794159990602886259U,
7212678433898152454U, 2933105860527989572U, 14594978649904627745U,
14177679044363798113U, 12176742446572697754U, 6364569523356538274U,
16716001465117549580U, 7427332547539658344U, 13451157428961778178U,
13470671439292824168U, 14426857945514077677U, 5939764529226844799U,
14347300826472120949U, 2041465696170965106U, 7798098743829532154U,
13171956973456043244U, 18236593193221364031U, 8149350929788359974U,
11936570011729054801U, 13042864289727585156U, 11911051690526917737U,
11792466550822869028U, 5981794705324482316U, 16406779455256359277U,
10722198138050885116U, 8878874700625925446U, 10789746635350659696U,
12188751068314879902U, 6952991988723463510U, 8655214120677782945U,
18277122291601228779U, 10135276750401732647U, 17013214853236557735U,
3544044491661506336U, 7637483428027845929U, 5301806499173418203U,
1534101427563769367U, 1383993655786492520U, 12475456912409478678U,
2554834275649251920U, 14506415064556034406U, 9838063362282724492U,
2449758835405158128U, 9357718492595438585U, 15806348368147251893U,
18194563017123726514U, 6089872301004121645U, 3255837569849134791U,
11962088332931191864U, 14293262028632301284U, 17840308675729353157U,
7178153646389378780U, 1241391272598079517U, 5487940136177241493U,
10654649640751110535U, 5568998332936970989U, 14626501281977855883U,
15722288015951976859U, 14075605759555249857U, 7175151490953833243U,
1094285656256348207U, 16726509009141958959U, 7942202204735717926U,
1786282484149594468U, 13740865428491922491U, 9219619342560343886U,
9039490016427611671U, 213153035923733121U, 10444498760262922950U,
13718349262725330964U, 8116327219997359068U, 10318408231970010400U,
7539913376372615979U, 520873968067150655U, 7076080361580830524U,
14686544390688766621U, 217656269077051426U, 10665157184775519914U,
6083867990133030572U, 2961626337165672173U, 15992482005151075182U,
8868367156601516066U, 10274876978154600114U, 5406881939417511996U,
6682797999524365187U, 13862452723631516736U, 15177396804400461908U,
5822680467240568859U, 8610181789144599891U, 16070538046475259142U,
12693113181486530105U, 13219991460424771835U, 2143538980979513362U,
12799689699448396666U, 18442240840556233310U, 18226085649196954652U,
7634481272592300392U, 5154700882831686894U, 12772670300528486833U,
17118290293480651528U, 8692741063622102157U, 1669198422163318528U,
8003746391164401433U, 4801947619155086306U, 13934504454084609622U,
261187522892461712U, 12798188621730623897U, 18368688032385367656U,
14621998048824537577U, 15501629591439379895U, 3263342958437998633U,
12329852373785520137U, 13866955956784835041U, 15398055228913058871U,
16634943268357820083U, 3455480906312912996U, 3297867745946772308U,
14021566961715430193U, 4527250396802669678U, 474340558816194833U,
4795943308283995232U, 13640293221401147004U, 4291581195112345029U,
7373293749699838679U, 10803256334810614613U, 12850726341852670793U,
2496292244656113950U, 11637855545892273877U, 16852599537434871510U,
14120638091088432911U, 9381735736079802880U, 16983193298881102366U,
2072988328244193244U, 9342707715417710900U, 15070820286438595347U,
600431087109107384U, 10974379194636710217U, 2789002399621803800U,
7533909065501524905U, 226662735383688037U, 11106474033800713842U,
9261649518657981403U, 11098968645211849999U, 8893885477803653130U,
11525274717059316242U, 11336138924619947416U, 2068485095090874939U,
9122049290905113936U, 4258557485321344123U, 5755131969940794279U,
5300305421455645435U, 1460548619392903712U, 16226650129123627062U,
1895861157547006566U, 663476351255563659U, 14063597137813067710U,
6586729025586908006U, 9155073000696114843U, 5876719265080388524U,
11258082883295763456U, 16690483143915412516U, 6176934808634942216U,
7521900443759342757U, 18084984343726314416U, 720517304530928861U,
16858603848305962584U, 14414849323771895530U, 5351342063859919563U,
3961344097202335968U, 9638420025818946287U, 11113979422389577684U,
9629413559512309676U, 10672662573364383756U, 6451632030987358844U,
2535320265318205930U, 13550228558334780896U, 18325156778569957370U,
12488966611869433594U, 3216809549187042811U, 10049715320488684845U,
12820704787497215424U, 1025236081238800858U, 13343079833282138848U,
8174869250990497038U, 13186967750633770929U, 525377201220468961U,
7296738786093427488U, 7052063118096466229U, 13509699459954916148U,
16339230957956584696U, 7412321770361930659U, 12715629347253121632U,
14323283582987756653U, 864620765437114633U, 5472929358999513808U,
9919121559042453989U, 6421610476631903475U, 1064264101900892838U,
15255452845724645868U, 9647426492125582898U, 11555296271414771612U,
12807195088037260508U, 363260807701009967U, 17799779577349488409U,
5192227825776006106U, 14611490504800128198U, 14986759934243320313U,
14928217903250182343U, 12059658384586421814U, 627450486029017216U,
12298329741712292000U, 12322346985196656295U, 13499191915930506768U,
15824361300760525114U, 630452641464562753U, 12445435358054023309U,
1083778112231938828U, 16211639351945899377U, 1160333075838350020U,
1516088494950496145U, 501359957736104665U, 6119893855359577015U,
4726893733266447883U, 10256864045541326893U, 4524248241367124141U,
327234942474463524U, 16034512181248712699U, 10927845785385754395U,
508865346324968508U, 6487657896213905288U, 4300587661418981640U,
7814610598725032607U, 13981037863335565444U, 2541324576189297004U,
13844439791018243514U, 14294763106350074053U, 17913861483900218812U,
10782241246761795854U, 11820987027460551629U, 7379298060570929753U,
11097467567494077231U, 8820332669632787476U, 7921187116686899168U,
756543169757475304U, 177127170697186678U, 8679231364162147240U,
1007223148625527637U, 12460446135231750993U, 1819306193940595374U,
15359027208250966891U, 14722570255915313064U, 1982923665177827136U,
4929539225165771625U, 1739749074898638646U, 11460728375195087198U,
8173368173272724269U, 13113414942462905274U, 15368033674557603502U,
15163887104940506992U, 5160705193702777968U, 13066881533211949452U,
13087896621260768210U, 14117635935652887374U, 9234630119738071571U,
9775018098136268217U, 17807284965938352251U, 5559991866630334379U,
14185184432952661955U, 12544506487427026027U, 5938263451509072031U,
14273748018301255294U, 16884122169508099648U, 15665247062676611657U,
11280599049062354983U, 17793775266478397335U, 4898016593092543487U,
195140103310459899U, 9561865062212535095U, 7362786205675429300U,
10288386677614555031U, 6068857212955302887U, 2226098255457015627U,
16845094148846007668U, 13752874050234104638U, 9808041807927269123U,
978702671987845036U, 11062942779985303556U, 7128618081702877420U,
17260892676669064531U, 15680257839854339342U, 12016127130771011529U,
16941163122783464849U, 13509699459954916U, 661975273537790891U,
13990044329642202055U, 2982641425214490931U, 17022221319543194346U,
3985361340686700263U, 10815264956552796760U, 13439148807219596030U,
12882248973925898931U, 4040901216244292696U, 13536718858874825980U,
17663181505032166479U, 16945666355936783155U, 234168123972551879U,
11474238074655042115U, 8835343446810515160U, 8656715198395555714U,
18350675099772094434U, 13739364350774149722U, 9146066534389478232U,
5435402416055194596U, 8080301354770812625U, 8553140835869234690U,
13275531335982364268U, 4864992883301542581U, 17023722397260967115U,
4058914148857565918U, 14419352556925213835U, 5572000488372516526U,
14773606898319587192U, 4483719142987259392U, 16788053195570642466U,
10957867339741209764U, 1979921509742281599U, 4782433608824040316U,
12978317947863356112U, 8748280939179694590U, 4390652324485347748U,
12227779088976971882U, 8865365001165970530U, 10127771361812868805U,
16645450812382229462U, 3970350563508972579U, 10079736874844140214U,
14291760950914528516U, 17766755867558487503U, 3574066046016961705U,
9108539591445159020U, 3596582211783553232U, 10211831714008143839U,
2317663996241154503U, 2885071373559260981U, 12241288788436926798U,
9527340274703761421U, 5671071617745519245U, 1181348163887168778U,
2545827809342615309U, 14065098215530840478U, 6660281833757773661U,
12759160601068531917U, 16456315019942860636U, 13149440807689451717U,
17133301070658379212U, 9428269145330758702U, 816586278468386042U,
3119239497531812861U, 5268782789382417297U, 18362683721514276582U,
14327786816141074959U, 1085279189949711597U, 16285192160116765032U,
4764420676210767094U, 12095684249812968257U, 2392717882129792926U,
6562711782102543711U, 7978228069962264369U, 3551549880250370178U,
8005247468882174201U, 4875500427325951961U, 17538592054457026697U,
10840783277754933824U, 14689546546124312158U, 364761885418782735U,
17873332385520354063U, 8796315426148423180U, 6744342185953048694U,
16878117858637008574U, 15371035829993149039U, 15310992721282238301U,
12368880394447612117U, 15779328969227342060U, 16870612470048144732U,
15003271789138820766U, 15737298793129704544U, 14811133841263906403U,
6322539347258900757U, 14656522836333311252U, 17193344179369289951U,
12370381472165384886U, 15852881777398207715U, 2027955996711010190U,
7136123470291741263U, 17628656717523392804U, 15253951768006873099U,
9573873683954717243U, 7951208671042354537U, 2227599333174788395U,
16918646957016873322U, 17356961650606521713U, 1940893489080189619U,
2870060596381533297U, 11505760706728270252U, 10379952418398693907U,
10555578511378107817U, 714512993659837787U, 16564392615622499966U,
18445242995991778847U, 18373191265538685961U, 14842656473337134541U,
7867148318847079503U, 16555386149315863355U, 18003926146966584920U,
15195409737013735129U, 6705314165290956714U, 14965744846194501555U,
13898478588858063179U, 16942664200501237618U, 87062507630820570U,
4266062873910207965U, 6122896010795122552U, 4873999349608179192U,
17465039246286161042U, 7236695677382516749U, 4109950791261840045U,
16920148034734646091U, 17430514458777387367U, 5544981089452606694U,
13449656351244005409U, 13397118631121958513U, 10822770345141660603U,
13806912848073924303U, 12455942902078432688U, 1598647769427998410U,
4546764407133715668U, 1430527065037448343U, 14755593965706313970U,
3601085444936871537U, 10432490138520740803U, 13129926797358405727U,
16177114564437125703U, 17915362561617991580U, 10855794054932661509U,
15425074627832968704U, 17958893815433401866U, 12988825491887765492U,
9263150596375754172U, 11172521453382715654U, 12497973078176070205U,
3658126398212236739U, 13227496849013635677U, 2511303021833841635U,
12373383627600930423U, 15999987393739939024U, 9236131197455844339U,
9848570906307133871U, 2964628492601217710U, 16139587621492806491U,
16076542357346350216U, 12987324414169992723U, 9189597788204888517U,
7568433853010298579U, 1918377323313598093U, 1766768473818548478U,
12784678922270668981U, 17706712758847576764U, 631953719182335522U,
12518988166224888963U, 4687865712604355903U, 8344491033098819874U,
3051691000232038281U, 1958906421693462841U, 3752694294431921152U,
17861323763778171916U, 8207892960781497944U, 14805129530392815329U,
6028328114575438138U, 240172434843642953U, 11768449307338504733U,
4804949774590631843U, 14081610070426340931U, 7469362723637295861U,
15510636057746016506U, 3704659807463192561U, 15507633902310470969U,
3557554191121461252U, 8299458701565636820U, 845106755106068643U,
4516742852778260298U, 18406214975329686867U, 16460818253096178942U,
13370099232202048681U, 9498819798066078820U, 4273568262499071808U,
6490660051649450824U, 4447693277760712949U, 15022785799469866756U,
16693485299350958053U, 6324040424976673525U, 14730075644504176906U,
2350687706032155409U, 4503233153318305382U, 17744239701791895976U,
2470773923453976886U, 10387457806987557749U, 10923342552232436089U,
288206921812371544U, 14122139168806205680U, 9455288544250668535U,
2140536825543967825U, 12652584083106665356U, 11234065639811399161U,
15513638213181562043U, 3851765423804923870U, 4269065029345753502U,
6270001627136853861U, 12082174550353013341U, 1730742608592002035U,
11019411526169893271U, 4995586644747773437U, 4976072634416727447U,
4019886128195473938U, 12506979544482706816U, 4099443247237430666U,
16405278377538586509U, 10648645329880019461U, 5274787100253508371U,
210150880488187584U, 10297393143921191641U, 6510174061980496814U,
5403879783981966459U, 6535692383182633878U, 6654277522886682587U,
12464949368385069299U, 2039964618453192338U, 7724545935658666499U,
9567869373083626169U, 7656997438358891919U, 6257993005394671713U,
11493752084986088105U, 9791529953031768670U, 169621782108322836U,
8311467323307818968U, 1433529220472993880U, 14902699582048045279U,
10809260645681705686U, 13144937574536133412U, 16912642646145782248U,
17062750417923059095U, 5971287161300072937U, 15891909798060299695U,
3940329009153517209U, 8608680711426827123U, 15996985238304393487U,
9089025581114113030U, 2640395705562299722U, 252181056585825101U,
12356871772705429970U, 15190906503860416824U, 6484655740778359751U,
4153482045077250331U, 606435397980198458U, 11268590427320172835U,
17205352801111472098U, 12958803937532310122U, 7792094432958441080U,
12877745740772580626U, 3820242791731695732U, 2724456057757574756U,
4371138314154301758U, 11271592582755718372U, 17352458417453203408U,
1720235064567592656U, 10504541868973833689U, 16660461589559957147U,
4705878645217629124U, 9227124731149207729U, 9407254057281939944U,
18233591037785818494U, 8002245313446628665U, 4728394810984220651U,
10330416853712192547U, 8128335841739541215U, 10906830697336935636U,
17925870105642400960U, 11370663712128721091U, 3760199683020784994U,
18229087804632500189U, 7781586888934031701U, 12362876083576521043U,
15485117736543879442U, 2454262068558476433U, 9578376917108035549U,
8171867095554951501U, 13039862134292039619U, 11763946074185186428U,
4584291350078034879U, 3269347269309089707U, 12624063606468982756U,
9836562284564951724U, 2376206027234292473U, 5753630892223021510U,
5226752613284779780U, 16303205092730038253U, 5647054374261154949U,
4503233153318305U, 220658424512596963U, 10812262801117251223U,
13292043190877864721U, 5674073773181064782U, 1328453780228900087U,
9754003010087449458U, 16777545651546233087U, 10442997682545150182U,
13644796454554465309U, 4512239619624941993U, 18185556550817089903U,
5648555451978927718U, 78056041324183959U, 3824746024885014038U,
2945114482270171720U, 15183401115271552982U, 6116891699924031478U,
4579788116924716574U, 3048688844796492744U, 1811800805351731532U,
14991263167396638619U, 15148876327762779307U, 4425177111994121422U,
13919493676906881937U, 17972403514893356782U, 13650800765425556383U,
4806450852308404611U, 14155162878597206586U, 11073450324009712936U,
7643487738898937002U, 5596017731856880822U, 15950451829053437665U,
6808888527817277738U, 1594144536274680105U, 4326105982621118704U,
9065008337629748735U, 1463550774828449249U, 16373755745465358371U,
9104036358291840715U, 3375923787270956268U, 17846312986600444231U,
7472364879072841398U, 15657741674087747815U, 10912835008208026710U,
18220081338325863578U, 7340270039908837773U, 9185094555051570212U,
7347775428497701616U, 9552858595905898485U, 6921469356650235373U,
7110605149089604199U, 16378258978618676676U, 9324694782804437679U,
14188186588388207492U, 12691612103768757336U, 13146438652253906180U,
16986195454316647903U, 2220093944585924553U, 16550882916162545049U,
17783267722453987956U, 4383146935896483905U, 11860015048122643609U,
9291671073013436772U, 12570024808629163091U, 7188661190413788159U,
1756260929794139099U, 12269809265074609399U, 10924843629950208858U,
361759729983237199U, 17726226769178622754U, 1588140225403589031U,
4031894749937656085U, 13095402009849632052U, 14485399976507215647U,
8808324047890605328U, 7332764651319973931U, 8817330514197241939U,
7774081500345167859U, 11995112042722192771U, 15911423808391345685U,
4896515515374770719U, 121587295139594245U, 5957777461840118021U,
15229934524522508804U, 8397028753220866770U, 5626039286212336191U,
17421507992470750757U, 5103664240427412767U, 10271874822719054577U,
5259776323075780686U, 17921366872489082654U, 11150005287616124127U,
11394680955613085386U, 4937044613754635467U, 2107513115752966919U,
11034422303347620956U, 5731114726456429983U, 4123460490721794962U,
17582123308272436982U, 12973814714710037807U, 8527622514667097626U,
12025133597077648140U, 17382479971808658777U, 3191291227984905747U,
8799317581583968717U, 6891447802294780003U, 5639548985672291107U,
18083483266008541648U, 646964496360063206U, 13254516247933545509U,
3835253568909423417U, 3459984139466231302U, 3518526170459369272U,
6387085689123129801U, 17819293587680534399U, 6148414331997259615U,
6124397088512895320U, 4947552157779044847U, 2622382772949026501U,
17816291432244988862U, 6001308715655528306U, 17362965961477612787U,
2235104721763652238U, 17286410997871201595U, 16930655578759055470U,
17945384115973446950U, 12326850218349974600U, 13719850340443103732U,
8189880028168224722U, 13922495832342427474U, 18119509131235088091U,
2412231892460838916U, 7518898288323797220U, 17937878727384583107U,
11959086177495646327U, 14146156412290569975U, 10632133474984519008U,
4465706210373986171U, 15905419497520254611U, 4602304282691308101U,
4151980967359477562U, 532882589809332803U, 7664502826947755761U,
6625757046248999986U, 11067446013138621862U, 7349276506215474384U,
9626411404076764139U, 10525556957022652447U, 17690200903952076311U,
18269616903012364937U, 9767512709547404375U, 17439520925084023978U,
5986297938477800622U, 16627437879768956241U, 3087716865458584724U,
3724173817794238551U, 16463820408531724479U, 13517204848543779990U,
16706994998810912969U, 6986015698514464417U, 10273375900436827346U,
5333329131246646341U, 3078710399151948113U, 3282856968769044623U,
13286038880006773647U, 5379862540497602163U, 5358847452448783405U,
4329108138056664241U, 9212113953971480044U, 8671725975573283398U,
639459107771199364U, 12886752207079217236U, 4261559640756889660U,
5902237586282525588U, 12508480622200479584U, 4172996055408296321U,
1562621904201451967U, 2781497011032939958U, 7166145024647196632U,
652968807231154280U, 13548727480617008128U, 18251603970399091716U,
8884879011497016520U, 11083957868034122315U, 8158357396094996584U,
12377886860754248728U, 16220645818252535988U, 1601649924863543947U,
4693870023475446977U, 8638702265782282492U, 17468041401721706579U,
7383801293724248059U, 11318125992006674195U, 1185851397040487084U,
2766486233855212273U, 6430616942938540086U, 1505580950926086766U,
18433234374249596699U, 17784768800171760724U, 4456699744067349560U,
15464102648495060684U, 1424522754166357269U, 14461382733022851352U,
7631479117156754855U, 5007595266489955585U, 5564495099783652684U,
14405842857465258919U, 4910025214834725635U, 783562568677385136U,
1501077717772768460U, 18212575949736999736U, 6972505999054509500U,
9611400626899036455U, 9790028875313995901U, 96068973937457181U,
4707379722935401893U, 9300677539320073383U, 13011341657654357019U,
10366442718938738990U, 9893603237840316925U, 5171212737727187347U,
13581751190408009034U, 1423021676448584500U, 14387829924851985697U,
4027391516784337780U, 12874743585337035089U, 3673137175389964423U,
13963024930722292223U, 1658690878138909149U, 7488876733968341851U,
16466822563967270016U, 13664310464885511299U, 5468426125846195503U,
9698463134529857025U, 14056091749224203867U, 6218964984732579733U,
9581379072543581085U, 8318972711896682810U, 1801293261327322153U,
14476393510200579036U, 8367007198865411401U, 4154983122795023099U,
679988206151064112U, 14872678027692589910U, 9338204482264392595U,
14850161861925998383U, 8234912359701407776U, 16129080077468397112U,
15561672700150290634U, 6205455285272624817U, 8919403799005790194U,
12775672455964032370U, 17265395909822382837U, 15900916264366936306U,
4381645858178711137U, 11786462239951777954U, 5687583472641019698U,
1990429053766690979U, 5297303266020099898U, 1313443003051172403U,
9018474928378792913U, 17630157795241165573U, 15327504576177738754U,
13177961284327134318U, 84060352195275033U, 4118957257568476656U,
17361464883759840018U, 2161551913592786583U, 13682323397498784521U,
6351059823896583358U, 16054026191579758689U, 11884032291607007904U,
10468516003747287246U, 14895194193459181437U, 10441496604827377414U,
13571243646383599654U, 908152019252524918U, 7605960795954617791U,
3757197527585239457U, 18081982188290768880U, 573411688189197552U,
9650428647561128435U, 11702401887756502921U, 1568626215072543041U,
3075708243716402576U, 3135751352427313314U, 6077863679261939498U,
2667415104482209555U, 1576131603661406883U, 3443472284570730849U,
2709445280579847071U, 3635610232445645212U, 12124204726450650858U,
3790221237376240363U, 1253399894340261664U, 6076362601544166729U,
2593862296311343900U, 16418788076998541425U, 11310620603417810352U,
818087356186158811U, 3192792305702678516U, 8872870389754834372U,
10495535402667197078U, 16219144740534763220U, 1528097116692678293U,
1089782423103029902U, 16505850584629361996U, 15576683477328018318U,
6940983366981281363U, 8066791655310857708U, 7891165562331443798U,
17732231080049713828U, 1882351458087051649U, 1501077717772768U};


const long flip[N] = {0, 256, 128, 384, 64, 320, 192, 448, 32, 288, 160, 416, 96, 352, 224,
480, 16, 272, 144, 400, 80, 336, 208, 464, 48, 304, 176, 432, 112, 368,
240, 496, 8, 264, 136, 392, 72, 328, 200, 456, 40, 296, 168, 424, 104,
360, 232, 488, 24, 280, 152, 408, 88, 344, 216, 472, 56, 312, 184, 440,
120, 376, 248, 504, 4, 260, 132, 388, 68, 324, 196, 452, 36, 292, 164,
420, 100, 356, 228, 484, 20, 276, 148, 404, 84, 340, 212, 468, 52, 308,
180, 436, 116, 372, 244, 500, 12, 268, 140, 396, 76, 332, 204, 460, 44,
300, 172, 428, 108, 364, 236, 492, 28, 284, 156, 412, 92, 348, 220, 476,
60, 316, 188, 444, 124, 380, 252, 508, 2, 258, 130, 386, 66, 322, 194,
450, 34, 290, 162, 418, 98, 354, 226, 482, 18, 274, 146, 402, 82, 338,
210, 466, 50, 306, 178, 434, 114, 370, 242, 498, 10, 266, 138, 394, 74,
330, 202, 458, 42, 298, 170, 426, 106, 362, 234, 490, 26, 282, 154, 410,
90, 346, 218, 474, 58, 314, 186, 442, 122, 378, 250, 506, 6, 262, 134,
390, 70, 326, 198, 454, 38, 294, 166, 422, 102, 358, 230, 486, 22, 278,
150, 406, 86, 342, 214, 470, 54, 310, 182, 438, 118, 374, 246, 502, 14,
270, 142, 398, 78, 334, 206, 462, 46, 302, 174, 430, 110, 366, 238, 494,
30, 286, 158, 414, 94, 350, 222, 478, 62, 318, 190, 446, 126, 382, 254,
510, 1, 257, 129, 385, 65, 321, 193, 449, 33, 289, 161, 417, 97, 353,
225, 481, 17, 273, 145, 401, 81, 337, 209, 465, 49, 305, 177, 433, 113,
369, 241, 497, 9, 265, 137, 393, 73, 329, 201, 457, 41, 297, 169, 425,
105, 361, 233, 489, 25, 281, 153, 409, 89, 345, 217, 473, 57, 313, 185,
441, 121, 377, 249, 505, 5, 261, 133, 389, 69, 325, 197, 453, 37, 293,
165, 421, 101, 357, 229, 485, 21, 277, 149, 405, 85, 341, 213, 469, 53,
309, 181, 437, 117, 373, 245, 501, 13, 269, 141, 397, 77, 333, 205, 461,
45, 301, 173, 429, 109, 365, 237, 493, 29, 285, 157, 413, 93, 349, 221,
477, 61, 317, 189, 445, 125, 381, 253, 509, 3, 259, 131, 387, 67, 323,
195, 451, 35, 291, 163, 419, 99, 355, 227, 483, 19, 275, 147, 403, 83,
339, 211, 467, 51, 307, 179, 435, 115, 371, 243, 499, 11, 267, 139, 395,
75, 331, 203, 459, 43, 299, 171, 427, 107, 363, 235, 491, 27, 283, 155,
411, 91, 347, 219, 475, 59, 315, 187, 443, 123, 379, 251, 507, 7, 263,
135, 391, 71, 327, 199, 455, 39, 295, 167, 423, 103, 359, 231, 487, 23,
279, 151, 407, 87, 343, 215, 471, 55, 311, 183, 439, 119, 375, 247, 503,
15, 271, 143, 399, 79, 335, 207, 463, 47, 303, 175, 431, 111, 367, 239,
495, 31, 287, 159, 415, 95, 351, 223, 479, 63, 319, 191, 447, 127, 383,
255, 511};

#elif (Q==19457)

#define normalization_factor 19419 // 1/N mod Q
#define oneoverQ 220741 // floor(2^32/Q)
#define oneover2Q 110370 // floor(2^32/2Q)


// v = [F(5)**i for i in range(1025)];
const long W[2*N+1] = {1, 5, 25, 125, 625, 3125, 15625, 297, 1485, 7425, 17668, 10512, 13646,
9859, 10381, 12991, 6584, 13463, 8944, 5806, 9573, 8951, 5841, 9748,
9826, 10216, 12166, 2459, 12295, 3104, 15520, 19229, 18317, 13757,
10414, 13156, 7409, 17588, 10112, 11646, 19316, 18752, 15932, 1832,
9160, 6886, 14973, 16494, 4642, 3753, 18765, 15997, 2157, 10785, 15011,
16684, 5592, 8503, 3601, 18005, 12197, 2614, 13070, 6979, 15438, 18819,
16267, 3507, 17535, 9847, 10321, 12691, 5084, 5963, 10358, 12876, 6009,
10588, 14026, 11759, 424, 2120, 10600, 14086, 12059, 1924, 9620, 9186,
7016, 15623, 287, 1435, 7175, 16418, 4262, 1853, 9265, 7411, 17598,
10162, 11896, 1109, 5545, 8268, 2426, 12130, 2279, 11395, 18061, 12477,
4014, 613, 3065, 15325, 18254, 13442, 8839, 5281, 6948, 15283, 18044,
12392, 3589, 17945, 11897, 1114, 5570, 8393, 3051, 15255, 17904, 11692,
89, 445, 2225, 11125, 16711, 5727, 9178, 6976, 15423, 18744, 15892,
1632, 8160, 1886, 9430, 8236, 2266, 11330, 17736, 10852, 15346, 18359,
13967, 11464, 18406, 14202, 12639, 4824, 4663, 3858, 19290, 18622,
15282, 18039, 12367, 3464, 17320, 8772, 4946, 5273, 6908, 15083, 17044,
7392, 17503, 9687, 9521, 8691, 4541, 3248, 16240, 3372, 16860, 6472,
12903, 6144, 11263, 17401, 9177, 6971, 15398, 18619, 15267, 17964,
11992, 1589, 7945, 811, 4055, 818, 4090, 993, 4965, 5368, 7383, 17458,
9462, 8396, 3066, 15330, 18279, 13567, 9464, 8406, 3116, 15580, 72, 360,
1800, 9000, 6086, 10973, 15951, 1927, 9635, 9261, 7391, 17498, 9662,
9396, 8066, 1416, 7080, 15943, 1887, 9435, 8261, 2391, 11955, 1404,
7020, 15643, 387, 1935, 9675, 9461, 8391, 3041, 15205, 17654, 10442,
13296, 8109, 1631, 8155, 1861, 9305, 7611, 18598, 15162, 17439, 9367,
7921, 691, 3455, 17275, 8547, 3821, 19105, 17697, 10657, 14371, 13484,
9049, 6331, 12198, 2619, 13095, 7104, 16063, 2487, 12435, 3804, 19020,
17272, 8532, 3746, 18730, 15822, 1282, 6410, 12593, 4594, 3513, 17565,
9997, 11071, 16441, 4377, 2428, 12140, 2329, 11645, 19311, 18727, 15807,
1207, 6035, 10718, 14676, 15009, 16674, 5542, 8253, 2351, 11755, 404,
2020, 10100, 11586, 19016, 17252, 8432, 3246, 16230, 3322, 16610, 5222,
6653, 13808, 10669, 14431, 13784, 10549, 13831, 10784, 15006, 16659,
5467, 7878, 476, 2380, 11900, 1129, 5645, 8768, 4926, 5173, 6408, 12583,
4544, 3263, 16315, 3747, 18735, 15847, 1407, 7035, 15718, 762, 3810,
19050, 17422, 9282, 7496, 18023, 12287, 3064, 15320, 18229, 13317, 8214,
2156, 10780, 14986, 16559, 4967, 5378, 7433, 17708, 10712, 14646, 14859,
15924, 1792, 8960, 5886, 9973, 10951, 15841, 1377, 6885, 14968, 16469,
4517, 3128, 15640, 372, 1860, 9300, 7586, 18473, 14537, 14314, 13199,
7624, 18663, 15487, 19064, 17492, 9632, 9246, 7316, 17123, 7787, 21,
105, 525, 2625, 13125, 7254, 16813, 6237, 11728, 269, 1345, 6725, 14168,
12469, 3974, 413, 2065, 10325, 12711, 5184, 6463, 12858, 5919, 10138,
11776, 509, 2545, 12725, 5254, 6813, 14608, 14669, 14974, 16499, 4667,
3878, 19390, 19122, 17782, 11082, 16496, 4652, 3803, 19015, 17247, 8407,
3121, 15605, 197, 985, 4925, 5168, 6383, 12458, 3919, 138, 690, 3450,
17250, 8422, 3196, 15980, 2072, 10360, 12886, 6059, 10838, 15276, 18009,
12217, 2714, 13570, 9479, 8481, 3491, 17455, 9447, 8321, 2691, 13455,
8904, 5606, 8573, 3951, 298, 1490, 7450, 17793, 11137, 16771, 6027,
10678, 14476, 14009, 11674, 19456, 19452, 19432, 19332, 18832, 16332,
3832, 19160, 17972, 12032, 1789, 8945, 5811, 9598, 9076, 6466, 12873,
5994, 10513, 13651, 9884, 10506, 13616, 9709, 9631, 9241, 7291, 16998,
7162, 16353, 3937, 228, 1140, 5700, 9043, 6301, 12048, 1869, 9345, 7811,
141, 705, 3525, 17625, 10297, 12571, 4484, 2963, 14815, 15704, 692,
3460, 17300, 8672, 4446, 2773, 13865, 10954, 15856, 1452, 7260, 16843,
6387, 12478, 4019, 638, 3190, 15950, 1922, 9610, 9136, 6766, 14373,
13494, 9099, 6581, 13448, 8869, 5431, 7698, 19033, 17337, 8857, 5371,
7398, 17533, 9837, 10271, 12441, 3834, 19170, 18022, 12282, 3039, 15195,
17604, 10192, 12046, 1859, 9295, 7561, 18348, 13912, 11189, 17031, 7327,
17178, 8062, 1396, 6980, 15443, 18844, 16392, 4132, 1203, 6015, 10618,
14176, 12509, 4174, 1413, 7065, 15868, 1512, 7560, 18343, 13887, 11064,
16406, 4202, 1553, 7765, 19368, 19012, 17232, 8332, 2746, 13730, 10279,
12481, 4034, 713, 3565, 17825, 11297, 17571, 10027, 11221, 17191, 8127,
1721, 8605, 4111, 1098, 5490, 7993, 1051, 5255, 6818, 14633, 14794,
15599, 167, 835, 4175, 1418, 7090, 15993, 2137, 10685, 14511, 14184,
12549, 4374, 2413, 12065, 1954, 9770, 9936, 10766, 14916, 16209, 3217,
16085, 2597, 12985, 6554, 13313, 8194, 2056, 10280, 12486, 4059, 838,
4190, 1493, 7465, 17868, 11512, 18646, 15402, 18639, 15367, 18464,
14492, 14089, 12074, 1999, 9995, 11061, 16391, 4127, 1178, 5890, 9993,
11051, 16341, 3877, 19385, 19097, 17657, 10457, 13371, 8484, 3506,
17530, 9822, 10196, 12066, 1959, 9795, 10061, 11391, 18041, 12377, 3514,
17570, 10022, 11196, 17066, 7502, 18053, 12437, 3814, 19070, 17522,
9782, 9996, 11066, 16416, 4252, 1803, 9015, 6161, 11348, 17826, 11302,
17596, 10152, 11846, 859, 4295, 2018, 10090, 11536, 18766, 16002, 2182,
10910, 15636, 352, 1760, 8800, 5086, 5973, 10408, 13126, 7259, 16838,
6362, 12353, 3394, 16970, 7022, 15653, 437, 2185, 10925, 15711, 727,
3635, 18175, 13047, 6864, 14863, 15944, 1892, 9460, 8386, 3016, 15080,
17029, 7317, 17128, 7812, 146, 730, 3650, 18250, 13422, 8739, 4781,
4448, 2783, 13915, 11204, 17106, 7702, 19053, 17437, 9357, 7871, 441,
2205, 11025, 16211, 3227, 16135, 2847, 14235, 12804, 5649, 8788, 5026,
5673, 8908, 5626, 8673, 4451, 2798, 13990, 11579, 18981, 17077, 7557,
18328, 13812, 10689, 14531, 14284, 13049, 6874, 14913, 16194, 3142,
15710, 722, 3610, 18050, 12422, 3739, 18695, 15647, 407, 2035, 10175,
11961, 1434, 7170, 16393, 4137, 1228, 6140, 11243, 17301, 8677, 4471,
2898, 14490, 14079, 12024, 1749, 8745, 4811, 4598, 3533, 17665, 10497,
13571, 9484, 8506, 3616, 18080, 12572, 4489, 2988, 14940, 16329, 3817,
19085, 17597, 10157, 11871, 984, 4920, 5143, 6258, 11833, 794, 3970,
393, 1965, 9825, 10211, 12141, 2334, 11670, 19436, 19352, 18932, 16832,
6332, 12203, 2644, 13220, 7729, 19188, 18112, 12732, 5289, 6988, 15483,
19044, 17392, 9132, 6746, 14273, 12994, 6599, 13538, 9319, 7681, 18948,
16912, 6732, 14203, 12644, 4849, 4788, 4483, 2958, 14790, 15579, 67,
335, 1675, 8375, 2961, 14805, 15654, 442, 2210, 11050, 16336, 3852,
19260, 18472, 14532, 14289, 13074, 6999, 15538, 19319, 18767, 16007,
2207, 11035, 16261, 3477, 17385, 9097, 6571, 13398, 8619, 4181, 1448,
7240, 16743, 5887, 9978, 10976, 15966, 2002, 10010, 11136, 16766, 6002,
10553, 13851, 10884, 15506, 19159, 17967, 12007, 1664, 8320, 2686,
13430, 8779, 4981, 5448, 7783, 1}; 


// for i in range(1025):                                              
//    v[i] = floor((2^(64)*((5**i)%19457)/19457))

const unsigned long W2[2*N+1] = {948077508028449U, 4740387540142249U, 23701937700711249U,
118509688503556249U, 592548442517781248U, 2962742212588906244U,
14813711062944531222U, 281579019884449649U, 1407895099422248247U,
7039475497111241237U, 16750633411846654569U, 9966190764395066381U,
12937465674556228676U, 9347096151652488532U, 9841992610843339431U,
12316474906797593927U, 6242142312859314788U, 12763967490587022326U,
8479605231806456784U, 5504538011613180689U, 9075945984356351833U,
8486241774362655934U, 5537720724394176439U, 9241859548261330582U,
9315809593887549682U, 9685559822018645182U, 11534310962674122678U,
2331322592241958545U, 11656612961209792728U, 2942832584920308794U,
14714162924601543972U, 18230582401879065016U, 17365935714557118617U,
13042702277947386625U, 9873279168608278281U, 12472907695622288177U,
7024306256982786037U, 16674787211204378569U, 9586959761183686382U,
11041310658499328679U, 18313065145077540166U, 17778349430549494367U,
15104770857909265372U, 1736877994708120396U, 8684389973540601983U,
6528461720283906687U, 14195564527709981823U, 15637590417421254271U,
4400975792268064891U, 3558134887630772843U, 17790674438153864217U,
15166395895931114622U, 2045003184817366646U, 10225015924086833231U,
14231591473015062923U, 15817725143946659770U, 5301649424895092390U,
8061503050765910335U, 3414027106410448443U, 17070135532052242218U,
11563701365423004628U, 2478274605986368295U, 12391373029931841477U,
6616632928530552537U, 14636420568943211073U, 17841870623587400517U,
15422376823098796121U, 3324907820655774143U, 16624539103278870719U,
9335719221556147132U, 9785107960361632431U, 12032051654389058927U,
4820026050816639791U, 5653386180373647339U, 9820186828158685081U,
12207445993374322177U, 5696997745742956039U, 10038244655005228581U,
13297735127607039675U, 11148443416906543529U, 401984863404062799U,
2009924317020313996U, 10049621585101569981U, 13354619778088746675U,
11432866669315078528U, 1824101125446737796U, 9120505627233688983U,
8709039988749341683U, 6651711796327605187U, 14811814907928474322U,
272098244804165149U, 1360491224020825747U, 6802456120104128737U,
15565536526811092071U, 4040706339217253892U, 1756787622376717846U,
8783938111883589233U, 7026202411998842937U, 16684267986284663069U,
9634363636585108882U, 11278330035506441179U, 1051417956403551048U,
5257089782017755240U, 7838704836379224585U, 2300036034477019695U,
11500180172385098478U, 2160668640796837546U, 10803343203984187730U,
17123227872501835418U, 11829163067670970628U, 3805583117226198292U,
581171512421439848U, 2905857562107199244U, 14529287810535996223U,
17306206831551326268U, 12744057862918424876U, 8380057093463469534U,
5006797319898244440U, 6587242525781670587U, 14489468555198801323U,
17107110554865351768U, 11748576479488552378U, 3402650176314107043U,
17013250881570535218U, 11279278113014469629U, 1056158343943693298U,
5280791719718466490U, 7957214524882780835U, 2892584476994800944U,
14462922384974004723U, 16974379703741368768U, 11084922223868637379U,
84378898214532049U, 421894491072660249U, 2109472455363301246U,
10547362276816506230U, 15843323236663427920U, 5429639888478933139U,
8701455368685114083U, 6613788696006467187U, 14622199406322784323U,
17770764810485266767U, 15066847757588127372U, 1547262493102430397U,
7736312465512151985U, 1788074180141656696U, 8940370900708283483U,
7808366356122314185U, 2148343633192467696U, 10741718165962338480U,
16815102682392589169U, 10288537117124739381U, 14549197438204593673U,
17405754969894313517U, 13241798554633361125U, 10868760552038150779U,
17450314612771650667U, 13464596769020046875U, 11982751623971579527U,
4573525898729242791U, 4420885419936662341U, 3657683025973760093U,
18288415129868800466U, 17655099354505795867U, 14488520477690772873U,
17102370167325209518U, 11724874541787841128U, 3284140487810550793U,
16420702439052753969U, 8316535900425563384U, 4689191354708713691U,
4999212699834016840U, 6549319425460532587U, 14299853053593111323U,
16159033046836901770U, 7008188939346302387U, 16594200623021960319U,
9184026820271595133U, 9026645953938872433U, 8239741622275258934U,
4305219963957191442U, 3079355746076405594U, 15396778730382027971U,
3196917357071933394U, 15984586785359666970U, 6135957631960128388U,
12233044086091090327U, 5824988209326796789U, 10678196972924432330U,
16497496717203058419U, 8700507291177085633U, 6609048308466324937U,
14598497468622073073U, 17652255121981710517U, 14474299315070346123U,
17031264354223075768U, 11369345476277172379U, 1506495160257207047U,
7532475801286035236U, 768890859011072948U, 3844454295055364742U,
775527401567272098U, 3877637007836360492U, 941440965472250848U,
4707204827361254241U, 5089280063096719590U, 6999656241774046337U,
16551537135160680069U, 8970709380965193883U, 7960058757406866185U,
2906805639615227694U, 14534028198076138473U, 17329908769252037518U,
12862567551421981126U, 8972605535981250783U, 7969539532487150685U,
2954209515016650194U, 14771047575083250972U, 68261580578048399U,
341307902890241999U, 1706539514451209996U, 8532697572256049984U,
5769999713861146689U, 10403254495596181830U, 15122784330561805922U,
1826945357970823146U, 9134726789854115733U, 8780145801851475433U,
7007240861838273937U, 16589460235481818069U, 9160324882570883883U,
8908136265435316183U, 7647193179757477685U, 1342477751368285197U,
6712388756841425987U, 15115199710497578322U, 1789022257649685146U,
8945111288248425733U, 7832068293823025435U, 2266853321696023945U,
11334266608480119729U, 1331100821271943797U, 6655504106359718987U,
14830776458089043322U, 366905995607010149U, 1834529978035050746U,
9172649890175253733U, 8969761303457165433U, 7955318369866723935U,
2883103701914516444U, 14415518509572582223U, 16737360326734256269U,
9899825338833074881U, 12605638546746271176U, 7687960512602701035U,
1546314415594401947U, 7731572077972009735U, 1764372242440945446U,
8821861212204727233U, 7215817913604532936U, 17632345494313113067U,
14374751176727358873U, 16533523662508139519U, 8880642017702491133U,
7509721941093352436U, 655121558047658948U, 3275607790238294743U,
16378038951191473719U, 8103218461119162135U, 3622604158176707443U,
18113020790883537216U, 16778127659579479619U, 10103662003059191631U,
13624821867876854924U, 12783877118255619776U, 8579153370149444034U,
6002278703328116938U, 11564649442931033078U, 2483014993526510545U,
12415074967632552727U, 6735142617034108787U, 15228969011460992321U,
2357868762466755145U, 11789343812333775728U, 3606486840540223793U,
18032434202701118966U, 16375194718667388369U, 8088997298498735385U,
3551498345074573693U, 17757491725372868467U, 15000482332026135872U,
1215435365292472897U, 6077176826462364488U, 11939140058602270827U,
4355468071882699291U, 3330596285703944843U, 16652981428519724219U,
9477930847760414632U, 10496166091382969930U, 15587342309495746421U,
4149735252640525642U, 2301932189493076595U, 11509660947465382978U,
2208072516198260045U, 11040362580991300229U, 18308324757537397916U,
17754647492848783117U, 14986261169405709122U, 1144329552190339147U,
5721647760951695739U, 10161494731048927081U, 13913985507825532174U,
14229695317999006023U, 15808244368866375270U, 5254245549493669890U,
7824483673758797835U, 2228930221374885945U, 11144651106874429729U,
383023313243493799U, 1915116566217468996U, 9575582831087344982U,
10984426008017621679U, 18028641892669005166U, 16356233168506819369U,
7994189547695890385U, 3077459591060348694U, 15387297955301743471U,
3149513481670510894U, 15747567408352554470U, 4950860746924565890U,
6307559660913277838U, 13091054230856837575U, 10115038933155533031U,
13681706518358561924U, 13068300370664154775U, 10001269632192119031U,
13112860013541491925U, 10224067846578804781U, 14226851085474920673U,
15794023206245948520U, 5183139736391536140U, 7468954608248129086U,
451284893821542199U, 2256424469107710995U, 11282122345538554979U,
1070379506564120048U, 5351897532820600240U, 8312743590393449584U,
4670229804548144691U, 4904404949031171840U, 6075280671446307588U,
11929659283521986328U, 4308064196481276792U, 3093576908696832344U,
15467884543484161721U, 3552446422582602143U, 17762232112913010717U,
15024184269726847122U, 1333945053796029147U, 6669725268980145737U,
14901882271191177072U, 722435061117678898U, 3612175305588394493U,
18060876527941972466U, 16517406344871655869U, 8800055429520072883U,
7106789000181261186U, 17087200927196754318U, 11649028341145565128U,
2904909484599170794U, 14524547422995853973U, 17282504893850615018U,
12625548174414868626U, 7787508650945688285U, 2044055107309338196U,
10220275536546690981U, 14207889535314351673U, 15699215455443103521U,
4709100982377311141U, 5098760838177004090U, 7047060117175468837U,
16788556512167792569U, 10155806266000756381U, 13885543182584678674U,
14087483691794738524U, 15097186237845037772U, 1698954894386982396U,
8494774471934911984U, 5580384212255456689U, 9455176987567731832U,
10382396790419555930U, 15018495804678676422U, 1305502728555175647U,
6527513642775878237U, 14190824140169839573U, 15613888479720543021U,
4282466103764508642U, 2965586445112991594U, 14827932225564957972U,
352684832986583399U, 1763424164932916996U, 8817120824664584983U,
7192115975903821686U, 17513835805809556817U, 13782202734209577624U,
13570781449919233274U, 12513675028467511526U, 7228142921208902786U,
17693970532334962317U, 14682876366836605122U, 18074149613054370766U,
16583771770433647369U, 9131882557330030383U, 8765924639231048683U,
6936135048736140187U, 16233931169971149320U, 7382679555017540136U,
19909627668597449U, 99548138342987249U, 497740691714936249U,
2488703458574681245U, 12443517292873406227U, 6877354243238376287U,
15940027142482329820U, 5913159417573442639U, 11119053014157661579U,
255032849659653049U, 1275164248298265247U, 6375821241491326238U,
13432362133747079575U, 11821578447606743028U, 3767660016905060293U,
391556010815749849U, 1957780054078749246U, 9788900270393746231U,
12051013204549627927U, 4914833801619484790U, 6127424934387872338U,
12190380598229810077U, 5611670770020395539U, 9611609776392426082U,
11164560734543027179U, 482571451586481049U, 2412857257932405245U,
12064286289662026227U, 4981199227181476290U, 6459252062197829838U,
13849516237279597574U, 13907348965269333024U, 14196512605218010273U,
15642330804961396521U, 4424677729968776141U, 3676644576134329093U,
18383222880671645466U, 18129138108520020866U, 16858714247761897868U,
10506594943971282880U, 15639486572437311171U, 4410456567348349391U,
3605538763032195343U, 18027693815160976716U, 16351492780966677119U,
7970487609995179135U, 2958949902556792444U, 14794749512783962222U,
186771269081604649U, 933856345408023248U, 4669281727040116241U,
4899664561491029590U, 6051578733745596338U, 11811149595018430078U,
3715515753963495543U, 130834696107926099U, 654173480539630498U,
3270867402698152493U, 16354337013490762469U, 7984708772615605885U,
3030055715658926194U, 15150278578294630972U, 1964416596634948396U,
9822082983174741981U, 12216926768454606677U, 5744401621144378539U,
10275264032012341081U, 14482832012642602173U, 17073927842084356018U,
11582662915583573628U, 2573082356789213295U, 12865411783946066476U,
8986826698601677533U, 8040645345589284435U, 3309738580527318943U,
16548692902636594719U, 8956488218344767133U, 7888952944304732435U,
2551276574104558945U, 12756382870522794726U, 8441682131485318784U,
5314922510007490690U, 8127868476327901835U, 3745854234220405943U,
282527097392478099U, 1412635486962390497U, 7063177434811952486U,
16869143100350210818U, 10558739206912847630U, 15900207887145134920U,
5714063140887468139U, 10123571630727789081U, 13724370006219842174U,
13281617809970556025U, 11067856828724125279U, 18445795996201523166U,
18442003686169409366U, 18423042136008840366U, 18328234385205995366U,
17854195631191770367U, 15484001861120645371U, 3633033010765020393U,
18165165053825101966U, 17038848974287303368U, 11407268576598310378U,
1696110661862897046U, 8480553309314485234U, 5509278399153322939U,
9099647922057063083U, 8604751462866212184U, 6130269166911957688U,
12204601760850236827U, 5682776583122529289U, 9967138841903094831U,
12942206062096370926U, 9370798089353199782U, 9960502299346895681U,
12909023349315375176U, 9204884525448221033U, 9130934479822001933U,
8761184251690906433U, 6912433111035428937U, 16115421481467593070U,
6790131112499758887U, 15503911488789242821U, 3732581149108007643U,
216161671830486599U, 1080808359152432998U, 5404041795762164990U,
8573464905101273334U, 5973836378087263438U, 11422437816726765578U,
1771956862505173046U, 8859784312525865233U, 7405433415210222936U,
133678928632011449U, 668394643160057248U, 3341973215800286243U,
16709866079001431219U, 9762354100168949632U, 11918282353425644928U,
4251179545999569792U, 2809153656288297344U, 14045768281441486724U,
14888609186078778772U, 656069635555687398U, 3280348177778436993U,
16401740888892184969U, 8221728149622718384U, 4215152600694488692U,
2629018929762891845U, 13145094648814459225U, 10385241022943641280U,
15032716967299103172U, 1376608541657309397U, 6883042708286546987U,
15968469467723183320U, 6055371043777710138U, 11830111145178999078U,
3810323504766340542U, 604873450122151098U, 3024367250610755494U,
15121836253053777472U, 1822204970430680896U, 9111024852153404483U,
8661636113347919184U, 6414692419320492688U, 13626718022892911824U,
12793357893335904276U, 8626557245550866534U, 6239298080335229438U,
12749746327966595576U, 8408499418704323034U, 5149008946102511940U,
7298300656803008086U, 18044759210305488816U, 16436819756689237619U,
8397122488607981634U, 5092124295620804940U, 7013877404394473087U,
16622642948262813819U, 9326238446475862632U, 9737704084960209932U,
11795032277381946428U, 3634929165781077293U, 18174645828905386466U,
17086252849688725868U, 11644287953605422878U, 2881207546898459544U,
14406037734492297723U, 16689956451332833769U, 9662805961825962382U,
11420541661710708678U, 1762476087424888546U, 8812380437124442733U,
7168414038203110436U, 17395326117306000567U, 13189654291691796375U,
10608039237330327030U, 16146708039232531920U, 6946563901324453137U,
16286075432912714069U, 7643400869725363885U, 1323516201207716197U,
6617581006038580987U, 14641160956483353323U, 17865572561288111767U,
15540886511602352371U, 3917456263173555392U, 1140537242158225347U,
5702686210791126739U, 10066686980246082081U, 13439946753811307175U,
11859501547927881028U, 3957275518510750292U, 1339633518844199847U,
6698167594220999237U, 15044093897395444572U, 1433493192139016397U,
7167465960695081986U, 17390585729765858317U, 13165952353991085125U,
10489529548826770780U, 15554159596714750671U, 3983821688735546892U,
1472364369968182847U, 7361821849840914236U, 18362365175495019566U,
18024849582636891366U, 16337271618346250369U, 7899381796893045385U,
2603420837046123695U, 13017104185230618476U, 9745288705024437532U,
11832955377703084428U, 3824544667386767292U, 675979263224284848U,
3379896316121424243U, 16899481580607121218U, 10710431608197399630U,
16658669893567894919U, 9506373173001268132U, 10638377717587237430U,
16298400440517083919U, 7705025907747213135U, 1631641391316962446U,
8158206956584812234U, 3897546635504957942U, 1040989103815238098U,
5204945519076190490U, 7577983521671400836U, 996429460937900948U,
4982147304689504740U, 6463992449737972088U, 13873218174980308824U,
14025858653772889274U, 14789061047735791522U, 158328943840751149U,
791644719203755748U, 3958223596018778742U, 1344373906384342097U,
6721869531921710487U, 15162603585899000822U, 2026041634656797646U,
10130208173283988231U, 13757552719000837924U, 13447531373875534775U,
11897424648249019028U, 4146891020116440292U, 2287711026872649845U,
11438555134363249228U, 1852543450687591296U, 9262717253437956482U,
9420098119770679182U, 10207002451434292681U, 14141524109752360173U,
15367388327633146021U, 3049965343327523644U, 15249826716637618221U,
2462157288349884645U, 12310786441749423227U, 6213699987618461288U,
12621755864382754826U, 7768547100785119285U, 1949247356506493196U,
9746236782532465982U, 11837695765243226678U, 3848246605087478542U,
794488951727841098U, 3972444758639205492U, 1415479719486475847U,
7077398597432379236U, 16940248913452344568U, 10914268272423516379U,
17677853214698478667U, 14602289778654186873U, 17671216672142279517U,
14569107065873191123U, 17505303108237300767U, 13739539246348297374U,
13357464010612832025U, 11447087831935505278U, 1895206938548871546U,
9476034692744357732U, 10486685316302685430U, 15539938434094323921U,
3912715875633413142U, 1116835304457514097U, 5584176522287570489U,
9474138537728300832U, 10477204541222400930U, 15492534558692901421U,
3675696498626300643U, 18378482493131503216U, 18105436170819309616U,
16740204559258341619U, 9914046501453501631U, 12676744359848404926U,
8043489578113369785U, 3323959743147745693U, 16619798715738728469U,
9312017283855435882U, 9666598271858076182U, 11439503211871277678U,
1857283838227733546U, 9286419191138667732U, 9538607808274235432U,
10799550893952073930U, 17104266322341266418U, 11734355316868125628U,
3331544363211973293U, 16657721816059866469U, 9501632785461125882U,
10614675779886526180U, 16179890752013527670U, 7112477465229431886U,
17115643252437607818U, 11791239967349832628U, 3615967615620508293U,
18079838078102541466U, 16612214095674500869U, 9274094183534297882U,
9476982770252386182U, 10491425703842827680U, 15563640371795035171U,
4031225564136969392U, 1709383746975295346U, 8546918734876476734U,
5841105526963280439U, 10758783561106850580U, 16900429658115149668U,
10715171995737541880U, 16682371831268606169U, 9624882861504824382U,
11230926160105018679U, 814398579396438548U, 4071992896982192742U,
1913220411201412096U, 9566102056007060482U, 10937022132616199179U,
17791622515661892667U, 15171136283471256872U, 2068705122518077896U,
10343525612590389480U, 14824139915532844172U, 333723282826014399U,
1668616414130071996U, 8343082070650359984U, 4821922205832696691U,
5662866955453931839U, 9867590703560107581U, 12444465370381434677U,
6882094630778518537U, 15963729080183041070U, 6031669106076998888U,
11711601456675442828U, 3217775062248559294U, 16088875311242796470U,
6657400261375775887U, 14840257233169327822U, 414309871008432649U,
2071549355042163246U, 10357746775210816230U, 14895245728634977922U,
689252348336683148U, 3446261741683415743U, 17231308708417078718U,
12369567247247187127U, 6507604015107280788U, 14091276001826852324U,
15116147788005606772U, 1793762645189827396U, 8968813225949136983U,
7950577982326581685U, 2859401764213805194U, 14297008821069025973U,
16144811884216475020U, 6937083126244168637U, 16238671557511291570U,
7406381492718251386U, 138419316172153699U, 692096580860768498U,
3460482904303842493U, 17302414521519212468U, 12725096312757855876U,
8285249342660624534U, 4532758565884019441U, 4217048755710545592U,
2638499704843176345U, 13192498524215881725U, 10622260399950753780U,
16217813852334665670U, 7302092966835121886U, 18063720760466057816U,
16531627507492082619U, 8871161242622206633U, 7462318065691929936U,
418102181040546449U, 2090510905202732246U, 10452554526013661230U,
15369284482649202921U, 3059446118407808144U, 15297230592039040721U,
2699176665356997145U, 13495883326784985725U, 12139184412796273777U,
5355689842852714040U, 8331705140554018584U, 4765037555350989691U,
5378443703045396840U, 8445474441517432584U, 5333884060168059690U,
8222676227130746834U, 4219892988234630942U, 2652720867463603095U,
13263604337318015475U, 10977789465461422529U, 17995459179888009416U,
16190319604601840620U, 7164621728170996636U, 17376364567145431567U,
13094846540888951375U, 10134000483316102031U, 13776514269161406924U,
13542339124678379775U, 12371463402263244027U, 6517084790187565287U,
14138679877228274823U, 15353167165012719271U, 2978859530225389894U,
14894297651126949472U, 684511960796540898U, 3422559803982704493U,
17112799019913522468U, 11777018804729405878U, 3544861802518374543U,
17724309012591872717U, 14834568768121157122U, 385867545767579149U,
1929337728837895746U, 9646688644189478732U, 11339955073528290429U,
1359543146512797297U, 6797715732563986487U, 15541834589110380821U,
3922196650713697642U, 1164239179858936597U, 5821195899294682989U,
10659235422763863330U, 16402688966400213419U, 8226468537162860634U,
4238854538395199942U, 2747528618266448094U, 13737643091332240474U,
13347983235532547525U, 11399683956534082778U, 1658187561541759046U,
8290937807708795234U, 4561200891124872941U, 4359260381914813091U,
3349557835864513843U, 16747789179322569219U, 9951969601774639631U,
12866359861454094926U, 8991567086141819783U, 8064347283289995685U,
3428248269030875193U, 17141241345154375968U, 11919230430933673378U,
4255919933539712042U, 2832855593989008594U, 14164277969945042973U,
15481157628596560021U, 3618811848144593643U, 18094059240722968216U,
16683319908776634619U, 9629623249044966632U, 11254628097805729929U,
932908267899994798U, 4664541339499973991U, 4875962623790318341U,
5933069045242040089U, 11218601152500648829U, 752773541374589298U,
3763867706872946493U, 372594460655180849U, 1862972303275904246U,
9314861516379521232U, 9680819434478502932U, 11510609024973411428U,
2212812903738402295U, 11064064518692011479U, 18426834446040954166U,
18347195935366564366U, 17949003381994615366U, 15958040615134870370U,
6003226780836145388U, 11569389830471175328U, 2506716931227221795U,
12533584656136108976U, 7327691059551890036U, 18191711224049898566U,
17171579825411286368U, 12070922832218225377U, 5014381939962472040U,
6625165626102808587U, 14679084056804491322U, 18055188062893801766U,
16488964019630802369U, 8657843803315805384U, 6395730869159923688U,
13531910272090066825U, 12319319139321679277U, 6256363475479741538U,
12835073303689156076U, 8835134297317125533U, 7282183339166524436U,
17964172622123070566U, 16033886815777146370U, 6382457784047525388U,
13465544846528075325U, 11987492011511721777U, 4597227836429954041U,
4539395108440218591U, 4250231468491541342U, 2804413268748155094U,
14022066343740775474U, 14770099497575222522U, 63521193037906149U,
317605965189530749U, 1588029825947653747U, 7940149129738268735U,
2807257501272240444U, 14036287506361202224U, 14841205310677356272U,
419050258548574899U, 2095251292742874496U, 10476256463714372480U,
15487794171152759171U, 3651994560925589393U, 18259972804627946966U,
17512887728301528367U, 13777462346669435374U, 13547079512218522025U,
12395165339963955277U, 6635594478691121537U, 14731228319746056072U,
18315909377601625516U, 17792570593169921117U, 15175876671011399122U,
2092407060218789146U, 10462035301093945730U, 15416688358050625421U,
3296465495414920643U, 16482327477074603219U, 8624661090534809634U,
6229817305254944938U, 12702342452565173076U, 8171480041697210534U,
3963912061066949442U, 1372816231625195597U, 6864081158125977987U,
15873661716920338320U, 5581332289763485139U, 9459917375107874082U,
10406098728120267180U, 15137005493182232672U, 1898051171072956896U,
9490255855364784482U, 10557791129404819180U, 15895467499604992670U,
5690361203186756889U, 10005061942224232831U, 13131821563702060925U,
10318875597381649780U, 14700889839489145672U, 18164216976317073516U,
17034108586747161118U, 11383566638897599129U, 1577600973359340797U,
7888004866796703985U, 2546536186564416695U, 12732680932822083476U,
8323172442981762534U, 4722374067489709441U, 5165126263738995590U,
7378887244985426336U, 948077508028449U}; 


const long flip[N] = {0, 256, 128, 384, 64, 320, 192, 448, 32, 288, 160, 416, 96, 352, 224,
480, 16, 272, 144, 400, 80, 336, 208, 464, 48, 304, 176, 432, 112, 368,
240, 496, 8, 264, 136, 392, 72, 328, 200, 456, 40, 296, 168, 424, 104,
360, 232, 488, 24, 280, 152, 408, 88, 344, 216, 472, 56, 312, 184, 440,
120, 376, 248, 504, 4, 260, 132, 388, 68, 324, 196, 452, 36, 292, 164,
420, 100, 356, 228, 484, 20, 276, 148, 404, 84, 340, 212, 468, 52, 308,
180, 436, 116, 372, 244, 500, 12, 268, 140, 396, 76, 332, 204, 460, 44,
300, 172, 428, 108, 364, 236, 492, 28, 284, 156, 412, 92, 348, 220, 476,
60, 316, 188, 444, 124, 380, 252, 508, 2, 258, 130, 386, 66, 322, 194,
450, 34, 290, 162, 418, 98, 354, 226, 482, 18, 274, 146, 402, 82, 338,
210, 466, 50, 306, 178, 434, 114, 370, 242, 498, 10, 266, 138, 394, 74,
330, 202, 458, 42, 298, 170, 426, 106, 362, 234, 490, 26, 282, 154, 410,
90, 346, 218, 474, 58, 314, 186, 442, 122, 378, 250, 506, 6, 262, 134,
390, 70, 326, 198, 454, 38, 294, 166, 422, 102, 358, 230, 486, 22, 278,
150, 406, 86, 342, 214, 470, 54, 310, 182, 438, 118, 374, 246, 502, 14,
270, 142, 398, 78, 334, 206, 462, 46, 302, 174, 430, 110, 366, 238, 494,
30, 286, 158, 414, 94, 350, 222, 478, 62, 318, 190, 446, 126, 382, 254,
510, 1, 257, 129, 385, 65, 321, 193, 449, 33, 289, 161, 417, 97, 353,
225, 481, 17, 273, 145, 401, 81, 337, 209, 465, 49, 305, 177, 433, 113,
369, 241, 497, 9, 265, 137, 393, 73, 329, 201, 457, 41, 297, 169, 425,
105, 361, 233, 489, 25, 281, 153, 409, 89, 345, 217, 473, 57, 313, 185,
441, 121, 377, 249, 505, 5, 261, 133, 389, 69, 325, 197, 453, 37, 293,
165, 421, 101, 357, 229, 485, 21, 277, 149, 405, 85, 341, 213, 469, 53,
309, 181, 437, 117, 373, 245, 501, 13, 269, 141, 397, 77, 333, 205, 461,
45, 301, 173, 429, 109, 365, 237, 493, 29, 285, 157, 413, 93, 349, 221,
477, 61, 317, 189, 445, 125, 381, 253, 509, 3, 259, 131, 387, 67, 323,
195, 451, 35, 291, 163, 419, 99, 355, 227, 483, 19, 275, 147, 403, 83,
339, 211, 467, 51, 307, 179, 435, 115, 371, 243, 499, 11, 267, 139, 395,
75, 331, 203, 459, 43, 299, 171, 427, 107, 363, 235, 491, 27, 283, 155,
411, 91, 347, 219, 475, 59, 315, 187, 443, 123, 379, 251, 507, 7, 263,
135, 391, 71, 327, 199, 455, 39, 295, 167, 423, 103, 359, 231, 487, 23,
279, 151, 407, 87, 343, 215, 471, 55, 311, 183, 439, 119, 375, 247, 503,
15, 271, 143, 399, 79, 335, 207, 463, 47, 303, 175, 431, 111, 367, 239,
495, 31, 287, 159, 415, 95, 351, 223, 479, 63, 319, 191, 447, 127, 383,
255, 511};

#elif (Q==25601)

#define normalization_factor 25551 // 1/N mod Q
#define oneoverQ 167765 // floor(2^32/Q)
#define oneover2Q 83882 // floor(2^32/2Q)

// v = [F(84)**i for i in range(1025)];
const long W[2*N+1] = {
1, 31, 961, 4190, 1885, 7233, 19415, 13042, 20287, 14473, 13446, 7210, 18702,
16540, 720, 22320, 693, 21483, 347, 10757, 654, 20274, 14070, 953, 3942,
19798, 24915, 4335, 6380, 18573, 12541, 4756, 19431, 13538, 10062, 4710,
18005, 20534, 22130, 20404, 18100, 23479, 11021, 8838, 17968, 19387, 12174,
18980, 25158, 11868, 9494, 12703, 9778, 21507, 1091, 8220, 24411, 14312, 8455,
6095, 9738, 20267, 13853, 19827, 213, 6603, 25486, 22036, 17490, 4569, 13634,
13038, 20163, 10629, 22287, 25271, 15371, 15683, 25355, 17975, 19604, 18901,
22709, 12752, 11297, 17394, 1593, 23782, 20414, 18410, 7488, 1719, 2087,
13495, 8729, 14589, 17042, 16282, 18323, 4791, 20516, 21572, 3106, 19483,
15150, 8832, 17782, 13621, 12635, 7670, 7361, 23383, 8045, 18986, 25344,
17634, 9033, 24013, 1974, 9992, 2540, 1937, 8845, 18185, 513, 15903, 6574,
24587, 19768, 23985, 1106, 8685, 13225, 359, 11129, 12186, 19352, 11089,
10946, 6513, 22696, 12349, 24405, 14126, 2689, 6556, 24029, 2470, 25368,
18378, 6496, 22169, 21613, 4377, 7682, 7733, 9314, 7123, 16005, 9736, 20205,
11931, 11447, 22044, 17738, 12257, 21553, 2517, 1224, 12343, 24219, 8360,
3150, 20847, 6232, 13985, 23919, 24661, 22062, 18296, 3954, 20170, 10846,
3413, 3399, 2965, 15112, 7654, 6865, 8007, 17808, 14427, 12020, 14206, 5169,
6633, 815, 25265, 15185, 9917, 215, 6665, 1807, 4815, 21260, 19035, 1262,
13521, 9535, 13974, 23578, 14090, 1573, 23162, 1194, 11413, 20990, 10665,
23403, 8665, 12605, 6740, 4132, 87, 2697, 6804, 6116, 10389, 14847, 25040,
8210, 24101, 4702, 17757, 12846, 14211, 5324, 11438, 21765, 9089, 148, 4588,
14223, 5696, 22970, 20843, 6108, 10141, 7159, 17121, 18731, 17439, 2988,
15825, 4156, 831, 160, 4960, 154, 4774, 19989, 5235, 8679, 13039, 20194,
11590, 876, 1555, 22604, 9497, 12796, 12661, 8476, 6746, 4318, 5853, 2236,
18114, 23913, 24475, 16296, 18757, 18245, 2373, 22361, 1964, 9682, 18531,
11239, 15596, 22658, 11171, 13488, 8512, 7862, 13313, 3087, 18894, 22492,
6025, 7568, 4199, 2164, 15882, 5923, 4406, 8581, 10001, 2819, 10586, 20954,
9549, 14408, 11431, 21548, 2362, 22020, 16994, 14794, 23397, 8479, 6839, 7201,
18423, 7891, 14212, 5355, 12399, 354, 10974, 7381, 24003, 1664, 382, 11842,
8688, 13318, 3242, 23699, 17841, 15450, 18132, 24471, 16172, 14913, 1485,
20434, 19030, 1107, 8716, 14186, 4549, 13014, 19419, 13166, 24131, 5632,
20986, 10541, 19559, 17506, 5065, 3409, 3275, 24722, 23953, 114, 3534, 7150,
16842, 10082, 5330, 11624, 1930, 8628, 11458, 22385, 2708, 7145, 16687, 5277,
9981, 2199, 16967, 13957, 23051, 23354, 7146, 16718, 6238, 14171, 4084, 24200,
7771, 10492, 18040, 21619, 4563, 13448, 7272, 20624, 24920, 4490, 11185,
13922, 21966, 15320, 14102, 1945, 9093, 272, 8432, 5382, 13236, 700, 21700,
7074, 14486, 13849, 19703, 21970, 15444, 17946, 18705, 16633, 3603, 9289,
6348, 17581, 7390, 24282, 10313, 12491, 3206, 22583, 8846, 18216, 1474, 20093,
8459, 6219, 13582, 11426, 21393, 23158, 1070, 7569, 4230, 3125, 20072, 7808,
11639, 2395, 23043, 23106, 25059, 8799, 16759, 7509, 2370, 22268, 24682,
22713, 12876, 15141, 8553, 9133, 1512, 21271, 19376, 11833, 8409, 4669, 16734,
6734, 3946, 19922, 3158, 21095, 13920, 21904, 13398, 5722, 23776, 20228,
12644, 7949, 16010, 9891, 25010, 7280, 20872, 7007, 12409, 664, 20584, 23680,
17252, 22792, 15325, 14257, 6750, 4442, 9697, 18996, 53, 1643, 25332, 17262,
23102, 24935, 4955, 25600, 25570, 24640, 21411, 23716, 18368, 6186, 12559,
5314, 11128, 12155, 18391, 6899, 9061, 24881, 3281, 24908, 4118, 25254, 14844,
24947, 5327, 11531, 24648, 21659, 5803, 686, 21266, 19221, 7028, 13060, 20845,
6170, 12063, 15539, 20891, 7596, 5067, 3471, 5197, 7501, 2122, 14580, 16763,
7633, 6214, 13427, 6621, 443, 13733, 16107, 12898, 15823, 4094, 24510, 17381,
1190, 11289, 17146, 19506, 15863, 5334, 11748, 5774, 25388, 18998, 115, 3565,
8111, 21032, 11967, 12563, 5438, 14972, 3314, 330, 10230, 9918, 246, 7626,
5997, 6700, 2892, 12849, 14304, 8207, 24008, 1819, 5187, 7191, 18113, 23882,
23514, 12106, 16872, 11012, 8559, 9319, 7278, 20810, 5085, 4029, 22495, 6118,
10451, 16769, 7819, 11980, 12966, 17931, 18240, 2218, 17556, 6615, 257, 7967,
16568, 1588, 23627, 15609, 23061, 23664, 16756, 7416, 25088, 9698, 19027,
1014, 5833, 1616, 24495, 16916, 12376, 25242, 14472, 13415, 6249, 14512,
14655, 19088, 2905, 13252, 1196, 11475, 22912, 19045, 1572, 23131, 233, 7223,
19105, 3432, 3988, 21224, 17919, 17868, 16287, 18478, 9596, 15865, 5396,
13670, 14154, 3557, 7863, 13344, 4048, 23084, 24377, 13258, 1382, 17241,
22451, 4754, 19369, 11616, 1682, 940, 3539, 7305, 21647, 5431, 14755, 22188,
22202, 22636, 10489, 17947, 18736, 17594, 7793, 11174, 13581, 11395, 20432,
18968, 24786, 336, 10416, 15684, 25386, 18936, 23794, 20786, 4341, 6566,
24339, 12080, 16066, 11627, 2023, 11511, 24028, 2439, 24407, 14188, 4611,
14936, 2198, 16936, 12996, 18861, 21469, 25514, 22904, 18797, 19485, 15212,
10754, 561, 17391, 1500, 20899, 7844, 12755, 11390, 20277, 14163, 3836, 16512,
25453, 21013, 11378, 19905, 2631, 4758, 19493, 15460, 18442, 8480, 6870, 8162,
22613, 9776, 21445, 24770, 25441, 20641, 25447, 20827, 5612, 20366, 16922,
12562, 5407, 14011, 24725, 24046, 2997, 16104, 12805, 12940, 17125, 18855,
21283, 19748, 23365, 7487, 1688, 1126, 9305, 6844, 7356, 23228, 3240, 23637,
15919, 7070, 14362, 10005, 2943, 14430, 12113, 17089, 17739, 12288, 22514,
6707, 3109, 19576, 18033, 21402, 23437, 9719, 19678, 21195, 17020, 15600,
22782, 15015, 4647, 16052, 11193, 14170, 4053, 23239, 3581, 8607, 10807, 2204,
17122, 18762, 18400, 7178, 17710, 11389, 20246, 13202, 25247, 14627, 18220,
1598, 23937, 25219, 13759, 16913, 12283, 22359, 1902, 7760, 10151, 7469, 1130,
9429, 10688, 24116, 5167, 6571, 24494, 16885, 11415, 21052, 12587, 6182,
12435, 1470, 19969, 4615, 15060, 6042, 8095, 20536, 22192, 22326, 879, 1648,
25487, 22067, 18451, 8759, 15519, 20271, 13977, 23671, 16973, 14143, 3216,
22893, 18456, 8914, 20324, 15620, 23402, 8634, 11644, 2550, 2247, 18455, 8883,
19363, 11430, 21517, 1401, 17830, 15109, 7561, 3982, 21038, 12153, 18329,
4977, 681, 21111, 14416, 11679, 3635, 10281, 11499, 23656, 16508, 25329,
17169, 20219, 12365, 24901, 3901, 18527, 11115, 11752, 5898, 3631, 10157,
7655, 6896, 8968, 21998, 16312, 19253, 8020, 18211, 1319, 15288, 13110, 22395,
3018, 16755, 7385, 24127, 5508, 17142, 19382, 12019, 14175, 4208, 2443, 24531,
18032, 21371, 22476, 5529, 17793, 13962, 23206, 2558, 2495, 542, 16802, 8842,
18092, 23231, 3333, 919, 2888, 12725, 10460, 17048, 16468, 24089, 4330, 6225,
13768, 17192, 20932, 8867, 18867, 21655, 5679, 22443, 4506, 11681, 3697,
12203, 19879, 1825, 5373, 12957, 17652, 9591, 15710, 591, 18321, 4729, 18594,
13192, 24937, 5017, 1921, 8349, 2809, 10276, 11344, 18851, 21159, 15904, 6605,
25548, 23958, 269, 8339, 2499, 666, 20646, 1};

// for i in range(1025):                                              
//    v[i] = floor((2^(64)*((31**i)%Q)/Q))

const unsigned long W2[2*N+1] = {
720547793981076U, 22336981613413386U, 692446430015814972U, 3019095256780712521U,
1358232591654330096U, 5211722193865129754U, 13989435420142609453U,
9397384329101205897U, 14617753096494108575U, 10428488222288127047U,
9688485637869560994U, 5195149594603564983U, 13475684843034101571U,
11917860512447013152U, 518794411666375421U, 16082626761657638063U,
499339621228886343U, 15479528258095476636U, 250030084511433710U,
7750932619854445011U, 471238257263624341U, 14608385975172354574U,
10138107461313753026U, 686682047663966356U, 2840399403873405432U,
14265405225237361934U, 17952448287038532811U, 3123574686907968683U,
4597094925599271095U, 13382734177610542641U, 9036389884316686333U,
3426925308174002089U, 14000964184846306685U, 9754776034915820076U,
7250151903037596514U, 3393780109650872548U, 12973463030629290920U,
14795728401607434587U, 15945722680801233438U, 14702057188389894581U,
13041915071057493232U, 16917741654881706276U, 7941157237465449332U,
6368201403204758297U, 12946802762251991072U, 13969260081911139298U,
8771948843925631083U, 13675997129760840969U, 18127541400975934516U,
8551461218967421529U, 6840880756056344792U, 9153118626941620803U,
7045516329546970653U, 15496821405151022483U, 786117643233354978U,
5922902866524452727U, 17589292198872070016U, 10312480027457173654U,
6092231598110005816U, 4391738804314664157U, 7016694417787727574U,
14603342140614487035U, 9981748590019859323U, 14286301111262813167U,
153476680117969395U, 4757777083657051260U, 18363881077401727763U,
15877991188167012203U, 12602380916729036278U, 3292182870699540695U,
9823948623138003465U, 9394502137925281589U, 14528405170040455030U,
7658702502224867158U, 16058848684456262523U, 18208963301695796214U,
11075540141283134170U, 11300351053005230186U, 18269489316390206680U,
12951846596809858610U, 14125618953205033001U, 13619073854036335888U,
16362919853516277006U, 9188425468846693574U, 8140028428604226577U,
12533208328506852888U, 1147832635811855619U, 17136067636457972599U,
14709262666329705350U, 13265284887191627094U, 5395461881330304382U,
1238621657853471318U, 1503783246038507645U, 9723792479774633766U,
6289661693660820907U, 10512071766389931976U, 12279575505025513793U,
11731959181599895293U, 13202597229115273397U, 3452144480963339783U,
14782758541315775202U, 15543657011759792487U, 2238021448105225081U,
14038432670133322687U, 10916299078813316158U, 6363878116440871835U,
12812780872571510754U, 9814581501816249465U, 9104121376950907568U,
5526601579834860391U, 5303952311494707606U, 16848569066659522887U,
5796807002577764257U, 13680320416524727431U, 18261563290656414833U,
12706139799062311362U, 6508708223031068307U, 17302514176867601381U,
1422361345318645947U, 7199713557458921126U, 1830191396711935514U,
1395701076941346098U, 6373245237762625836U, 13103161633545884775U,
369641018312292487U, 11458871567681067120U, 4736881197631600028U,
17716108610612739564U, 14243788791417929625U, 17282338838636131225U,
796925860143071133U, 6257957590725653520U, 9529244575399742983U,
258676658039206633U, 8018976399215405645U, 8780595417453404007U,
13944040909121801604U, 7990154487456162566U, 7887116152916868559U,
4692927782198754332U, 16353552732194523006U, 8898044707872319554U,
17584968912108183554U, 10178458137776693337U, 1937553018015115983U,
4723911337339940642U, 17314042941571298612U, 1779753051133260126U,
18278856437711960681U, 13242227357784232631U, 4680678469701076024U,
15973824044766495440U, 15573199471313016642U, 3153837694255173915U,
5535248153362633315U, 5571996090855668241U, 6711182153139750937U,
5132461936527211287U, 11532367442667136971U, 7015253322199765420U,
14558668177387660263U, 8596855729988229378U, 8248110597701388123U,
15883755570518860818U, 12781076769636343368U, 8831754310826060472U,
15529966603674152024U, 1813618797450370744U, 881950499832838216U,
8893721421108433092U, 17450947022427703237U, 6023779557681803504U,
2269725551040392468U, 15021259861123511680U, 4490453852090071703U,
10076860898825361483U, 17234782684233380145U, 17769429147367339260U,
15896725430810520204U, 13183142438677784319U, 2849045977401178355U,
14533449004598322569U, 7815061373518760861U, 2459229620857415712U,
2449141951741680635U, 2136424209153893228U, 10888918262642035233U,
5515072815131163160U, 4946560605680093427U, 5769426186406483332U,
12831515115215018756U, 10395343023764997506U, 8660984483652545229U,
10236101961295179495U, 3724511547088186879U, 4779393517476483569U,
587246452094577733U, 18204640014931909752U, 10941518251602653852U,
7145672472910340352U, 154917775705931549U, 4802451046883878032U,
1302029863723806092U, 3469437628018885630U, 15318846100037696471U,
13715627258429800203U, 909331316004119141U, 9742526722418141767U,
6870423215609568948U, 10068934873091569637U, 16989075886485832897U,
10152518417193374566U, 1133421679932234080U, 16689328004189704875U,
860334066013405907U, 8223611972706031506U, 15124298195662805688U,
7684642222808185929U, 16862980022539144426U, 6243546634846031981U,
9082504943131475259U, 4856492131432458805U, 2977303484729810057U,
62687658076353696U, 1943317400366964599U, 4902607190247247732U,
4406870307988266774U, 7485771031669408684U, 10697973097237049835U,
18042516761286167433U, 5915697388584641958U, 17365922382737936154U,
3388015727299023932U, 12794767177721983830U, 9256156961480914810U,
10239704700265084880U, 3836196455155253810U, 8241625667555558430U,
15682722735998140343U, 6549058899494008618U, 106641073509199392U,
3305873278785181157U, 10248351273792857803U, 4104240234516214444U,
16550982827745338096U, 15018377669947587372U, 4401105925636418158U,
7307075178762101595U, 5158401657110530058U, 12336498780750018874U,
13496580729059552803U, 12565632979236001352U, 2152996808415457998U,
11402668839750543116U, 2994596631785355904U, 598775216798274965U,
115287647036972315U, 3573917058146141791U, 110964360273085854U,
3439895168465661474U, 14403029853887747636U, 3772067701490937959U,
6253634303961767058U, 9395222685719262666U, 14550742151653868416U,
8351148932240682130U, 631199867527423429U, 1120451819640574694U,
16287262335148263924U, 6843042399438288023U, 9220129571781860961U,
9122855619594415570U, 6107363101783608433U, 4860815418196345267U,
3111325374410290374U, 4217366238171243529U, 1611144867341688114U,
13052002740173228310U, 17230459397469493683U, 17635407257686858943U,
11742046850715630371U, 13515314971703060804U, 13146394501184749393U,
1709859915117095659U, 16112169221210862219U, 1415155867378835177U,
6976343741324787264U, 13352471170263337408U, 8098236656553324112U,
11237663394928876489U, 16326171916023242080U, 8049239406562610878U,
9718748645216766227U, 6133302822366927204U, 5664946756279227171U,
9592652781270077757U, 2224331040019584619U, 13614030019478468350U,
16206560982222383303U, 4341300458735988769U, 5453105704848790540U,
3025580186926542214U, 1559265426175050572U, 11443740064007464503U,
4267804583749918918U, 3174733580280625148U, 6183020620151621515U,
7206198487604750818U, 2031224231232655990U, 7627718947083680848U,
15098358475079486916U, 6880510884725304026U, 10381652615679357043U,
8236581832997690891U, 15526363864704246639U, 1701933889383303813U,
15866462423463314971U, 12244989210914422099U, 10659784064156052756U,
16858656735775257964U, 6109524745165551664U, 4927826363036585426U,
5188664664457735291U, 13274652008513381095U, 5685842642304678403U,
10240425248059065957U, 3858533436768667196U, 8934072097571373402U,
255073919069301248U, 7907291491148338714U, 5318363267374329146U,
17295308698927790611U, 1198991529184512085U, 275249257300771404U,
8532726976323913528U, 6260119234107596751U, 9596255520239983142U,
2336015948086651550U, 17076262169557543211U, 12855293192416394296U,
11132463417007639251U, 13064972600464887695U, 17632525066510934635U,
11652698924261976826U, 10745529251639800915U, 1070013474061899306U,
14723673622209326890U, 13712024519459894818U, 797646407937052210U,
6280294572339066906U, 10221691005415557955U, 3277771914819919155U,
9377208990869735741U, 13992317611318533761U, 9486732255554859442U,
17387538816557368463U, 4058125175701425518U, 15121416004486881380U,
7595294296354532384U, 14093194302475884538U, 12613909681432733510U,
3649574576514154874U, 2456347429681491404U, 2359794025288027090U,
17813382562800184955U, 17259281309228736762U, 82142448513842775U,
2546415903929126026U, 5151916726964700365U, 12135465946229298399U,
7264562858917218053U, 3840519741919140272U, 8375647557236038747U,
1390657242383478560U, 6216886366468732133U, 8256036623435179970U,
16129462368266408067U, 1951243426100756446U, 5148313987994794980U,
12023781038162231468U, 3802330708838143192U, 7191787531725129279U,
1584484598964388266U, 12225534420476933020U, 10056685560593891328U,
16609347199057805331U, 16827673180634071655U, 5149034535788776057U,
12046118019775644854U, 4494777138853958164U, 10210882788505841801U,
2942717190618718362U, 17437256614342062775U, 5599376907026949166U,
7559987454449459613U, 12998682203418628614U, 15577522758076903104U,
3287859583935654233U, 9689926733457523148U, 5239823557830391756U,
14860577703065731515U, 17956051026008438196U, 3235259594975035614U,
8059327075678345956U, 10031466387804553634U, 15827552842588336814U,
11038792203790099244U, 10161164990721147489U, 1401465459293194714U,
6551941090669932926U, 195988999962852936U, 6075658998848441046U,
3877988227206156275U, 9537170601133534830U, 504383455786753881U,
15635887129389370339U, 5097155094622138515U, 10437855343609881047U,
9978866398843935015U, 14196953184809159622U, 15830435033764261122U,
11128140130243752789U, 12930950710784407378U, 13477846486416044802U,
11984871457287253311U, 2596133701713820337U, 6693168458290224013U,
4574037396191876632U, 12667950765981314283U, 5324848197520158839U,
17496341533448511087U, 7431009399326846834U, 9000362494617632484U,
2310076227503332779U, 16272130831474661307U, 6373965785556606913U,
13125498615159298161U, 1062087448328107459U, 14477966824461779642U,
6095113789285930124U, 4481086730768317702U, 9786480137850987463U,
8232979094027785506U, 15414678956637179708U, 16686445813013780568U,
770986139559752362U, 5453826252642771617U, 3047917168539955600U,
2251711856190865544U, 14462835320788177025U, 5626037175404249014U,
8386455774145754902U, 1725711966584679353U, 16603582816705956716U,
16648977327726764565U, 18056207169371807895U, 6340100039239496295U,
12075660479328869010U, 5410593385003906999U, 1707698271735152428U,
16045158276370622061U, 17784560651040941876U, 16365802044692201314U,
9277773395300347119U, 10909814148667486466U, 6162845281920151360U,
6580763002429176005U, 1089468264499388384U, 15326772125771488317U,
13961334056177347451U, 8526242046178083835U, 6059086399586876275U,
3364237650097648392U, 12057646784479342085U, 4852168844668572344U,
2843281595049329740U, 14354753151691015479U, 2275489933392241084U,
15199955714030818770U, 10030025292216591480U, 15782878879361510042U,
9653899343758469300U, 4122974477159722446U, 17131744349694086138U,
14575240776649225033U, 9110606307096737261U, 5727634414355580867U,
11535970181637042356U, 7126938230266832351U, 18020900327466735124U,
5245587940182240372U, 15039273555973038605U, 5048878392425406358U,
8941277575511184172U, 478443735203435110U, 14831755791306488436U,
17062571761471902748U, 12430890541761539958U, 16422725320416706395U,
11042394942760004629U, 10272849898788214420U, 4863697609372269575U,
3200673300863943919U, 6987151958234503418U, 13687525894464538201U,
38189033080997079U, 1183860025510909468U, 18252916717128641909U,
12438096019701350728U, 16646095136550840257U, 17966859242918154351U,
3570314319176236407U, 18446023525915570539U, 18424407092096138229U,
17754297643693736643U, 15427648816928839094U, 17088511482055221519U,
13235021879844421861U, 4457308653566942162U, 9049359744608345718U,
3828990977215443040U, 8018255851421424568U, 8758258435839990621U,
13251594479105986632U, 4971059230675450044U, 6528883561262538463U,
17927949662043176194U, 2364117312051913552U, 17947404452480665272U,
2967215815614074979U, 18196713989198117905U, 10695811453855106604U,
17975505816445927274U, 3838358098537197041U, 8308636612395798589U,
17760062026045585259U, 15606344669836146183U, 4181338848472189681U,
494295786671018804U, 15323169386801582932U, 13849649148110280520U,
5064009896099008974U, 9410354189392865282U, 15019818765535549526U,
4445779888863244930U, 8691968038793731539U, 11196592170671955101U,
15052963964058679067U, 5473281043080260695U, 3651015672102117028U,
2501021392908318177U, 3744686885319657034U, 5404829002652058383U,
1529002418827845339U, 10505586836244102283U, 12078542670504793318U,
5499941311457560543U, 4477483991798412317U, 9674795229783920532U,
4770746943948710646U, 319202672733617099U, 9895282854742130086U,
11605863317653206823U, 9293625446767930812U, 11401227744162580962U,
2949922668558529132U, 17660626430476196637U, 12523841207185098888U,
857451874837481599U, 8134264046252377961U, 12354512475599545799U,
14055005269394887458U, 11430049655921824041U, 3843401933095064580U,
8464995483689692292U, 4160442962446738448U, 18293267393591582220U,
13688966990052500355U, 82862996307823852U, 2568752885542539412U,
5844363156980515337U, 15154561203010010920U, 8622795450571548150U,
9052241935784270026U, 3918338903669096585U, 10788041571484684457U,
2387895389253289092U, 237780772013755401U, 7371203932426417445U,
7146393020704321429U, 177254757319344935U, 5494897476899693005U,
4321125120504518614U, 4827670219673215727U, 2083824220193274609U,
9258318604862858041U, 10306715645105325038U, 5913535745202698727U,
17298911437897695996U, 1310676437251579016U, 3737481407379846265U,
5181459186517924521U, 13051282192379247233U, 17208122415856080297U,
16942960827671043970U, 8722951593934917849U, 12157082380048730708U,
7934672307319619639U, 6167168568684037822U, 6714784892109656322U,
5244146844594278218U, 14994599592746211832U, 3663985532393776413U,
2903087061949759128U, 16208722625604326534U, 4408311403576228928U,
7530444994896235457U, 12082865957268679780U, 5633963201138040861U,
8632162571893302150U, 9342622696758644047U, 12920142493874691224U,
13142791762214844009U, 1598175007050028728U, 12649937071131787358U,
4766423657184824184U, 185180783053136782U, 5740604274647240253U,
11938035850678483308U, 1144229896841950234U, 17024382728390905668U,
11247030516250630489U, 16616552676997616101U, 17051042996768205517U,
12073498835946925779U, 5343582440163666840U, 18077103055397259128U,
6987872506028484495U, 13709862876077951587U, 730635463096812051U,
4202955282291621990U, 1164405235073420390U, 17649818213566480482U,
12188786482983898095U, 8917499498309808632U, 18188067415670344982U,
10427767674494145970U, 9666148656256147608U, 4502703164587750011U,
10456589586253389049U, 10559627920792683056U, 13753816291510797283U,
2093191341515028609U, 9548699365837232061U, 861775161601368061U,
8268285935932858278U, 16509191055694435632U, 13722832736369610973U,
1132701132138253003U, 16666991022576291489U, 167887635997590934U,
5204516715925318984U, 13766065604008475591U, 2472920028943056175U,
2873544602396534973U, 15292906379454377700U, 12911495920346918300U,
12874747982853883374U, 11735561920569800678U, 13314282137182340328U,
6914376631042414644U, 11431490751509786195U, 3888075896321891352U,
9849888343721322237U, 10198633476008163492U, 2562988503190690797U,
5665667304073208247U, 9614989762883491143U, 2916777470035399591U,
16633125276259180871U, 17564793573876713399U, 9553022652601118523U,
995797051281848378U, 12422964516027748111U, 16177018522669159147U,
3425484212586039935U, 13956290221619479912U, 8369883174884190132U,
1211961389476171470U, 677314926342212355U, 2550018642899031411U,
5263601635031767296U, 15597698096308373260U, 3913295069111229046U,
10631682700190790754U, 15987514452852135903U, 15997602121967870980U,
16310319864555658387U, 7557825811067516382U, 12931671258578388455U,
13500183468029458188U, 12677317887303068283U, 5615228958494532859U,
8051401049944554109U, 9785759590057006386U, 8210642112414372120U,
14722232526621364736U, 13667350556233068046U, 17859497621614973882U,
242104058777641863U, 7505225822106897763U, 11301071600799211263U,
18291826298003620066U, 13644293026825673583U, 17144714209985745523U,
14977306445690665985U, 3127897973671855144U, 4731116815279751412U,
17537412757705432474U, 8704217351291409848U, 11576320858099982667U,
8377809200617981978U, 1457668187223718718U, 8294225656516177049U,
17313322393777317535U, 1757416069519846740U, 17586410007696145708U,
10223132101003520109U, 3322445878046745927U, 10762101850901365686U,
1583764051170407189U, 12203197438863519634U, 9364239130578076356U,
13590251942277092810U, 15469440588979741558U, 18384056415633197919U,
16503426673342587016U, 13544136883462303883U, 14039873765721284841U,
10960973042040142931U, 7748770976472501780U, 404227312423384182U,
12531046685124909657U, 1080821690971615461U, 15058728346410527683U,
5651976895987567785U, 9190587112228636805U, 8207039373444466735U,
14610547618554297805U, 10205118406153993185U, 2764021337711411272U,
11897685174215542997U, 18340103000200352223U, 15140870794924370458U,
8198392799916693812U, 14342503839193337171U, 1895761245964213519U,
3428366403761964243U, 14045638148073133457U, 11139668894947450020U,
13288342416599021557U, 6110245292959532741U, 4950163344649998812U,
5881111094473550263U, 16293747265294093617U, 7044075233959008499U,
15452147441924195711U, 17847968856911276650U, 18331456426672579300U,
14872827015563409824U, 18335779713436465761U, 15006848905243890141U,
4043714219821803979U, 14674676372218613656U, 12193109769747784557U,
9051521387990288949U, 3896001922055683199U, 10095595141468869485U,
17815544206182128186U, 17326292254068976921U, 2159481738561287691U,
11603701674271263592U, 9226614501927690654U, 9323888454115136045U,
12339380971925943182U, 13585928655513206348U, 15335418699299261241U,
14229377835538308086U, 16835599206367863501U, 5394741333536323305U,
1216284676240057932U, 811336816022692672U, 6704697222993921244U,
4931429102006490811U, 5300349572524802222U, 16736884158592455956U,
2334574852498689396U, 17031588206330716438U, 11470400332384764351U,
5094272903446214207U, 10348507417156227503U, 7209080678780675126U,
2120572157686309535U, 10397504667146940737U, 8727995428492785388U,
12313441251342624411U, 12781797317430324444U, 8854091292439473858U,
16222413033689966996U, 4832714054231083265U, 2240183091487168312U,
14105443614973562846U, 12993638368860761075U, 15421163886783009401U,
16887478647534501043U, 7003004009702087112U, 14178939489959632697U,
15272010493428926467U, 12263723453557930100U, 11240545586104800797U,
16415519842476895625U, 10819025126625870767U, 3348385598630064699U,
11566233188984247589U, 8065091458030194572U, 10210162240711860724U,
2920380209005304976U, 16744810184326247802U, 2580281650246236644U,
6201754862795129516U, 7786960009553498859U, 1588087337934293651U,
12337219328543999951U, 13518917710672966189U, 13258079409251816324U,
5172092065196170520U, 12760901431404873212U, 8206318825650485658U,
14588210636940884419U, 9512671976138178213U, 18191670154640250367U,
10539452582561212901U, 13128380806335222469U, 1151435374781761004U,
17247752544525039530U, 18171494816408780211U, 9914017097385638087U,
12186624839601954864U, 8850488553469568473U, 16110728125622900065U,
1370481904152008404U, 5591450881293157319U, 7314280656701912364U,
5381771473244663920U, 814219007198616980U, 6794045149447574789U,
7701214822069750700U, 17376730599647652309U, 3723070451500224725U,
4734719554249656797U, 17649097665772499405U, 12166449501370484709U,
8225053068293993660U, 15168972158889632460U, 9069535082839815874U,
4454426462391017854U, 8960011818154692173U, 1059205257152183152U,
14388618898008126097U, 3325328069222670235U, 10851449777355019231U,
4353549771233667077U, 5832834392276818105U, 14797169497195396741U,
15990396644028060211U, 16086950048421524525U, 633361510909366660U,
1187462764480814853U, 18364601625195708840U, 15900328169780425589U,
13294827346744851250U, 6311278127480253216U, 11182181214792333562U,
14606224331790411343U, 10071096516473512868U, 17056086831326073055U,
12229857707240819482U, 10190707450274371645U, 2317281705443143548U,
16495500647608795169U, 13298430085714756635U, 6422963035547320147U,
14644413364871408423U, 11254956541984422336U, 16862259474745163349U,
6221209653232618595U, 8390058513115660287U, 1837396874651746284U,
1619070893075479960U, 13297709537920775558U, 6400626053933906761U,
13951966934855593451U, 8235861285203709814U, 15504026883090833253U,
1009487459367488840U, 12847367166682602449U, 10886756619260092002U,
5448061870290923001U, 2869221315632648511U, 15158884489773897382U,
8756817340252028467U, 13206920515879159859U, 3586166370643820100U,
490693047701113419U, 15211484478734516001U, 10387416998031205659U,
8415277685904997981U, 2619191231121214801U, 7407951869919452371U,
8285579082988404126U, 17045278614416356901U, 11894802983039618689U,
18250755073746698679U, 12371085074861110569U, 14568755846503395340U,
8909573472576016785U, 17942360617922797734U, 2810856944320181276U,
13349588979087413100U, 8008888730099670568U, 8467877674865616600U,
4249790888900391993U, 2616309039945290493U, 7318603943465798826U,
5515793362925144237U, 4968897587293506813U, 6461872616422298304U,
15850610371995731278U, 11753575615419327602U, 13872706677517674983U,
5778793307728237332U, 13121895876189392776U, 950402540261040528U,
11015734674382704781U, 9446381579091919131U, 16136667846206218836U,
2174613242234890308U, 12072778288152944702U, 5321245458550253454U,
17384656625381444156U, 3968777249247771973U, 12351630284423621491U,
13965657342941233913U, 8660263935858564152U, 10213764979681766109U,
3032065117072371907U, 1760298260695771047U, 17675757934149799253U,
12992917821066779998U, 15398826905169596015U, 16195032217518686071U,
3983908752921374590U, 12820706898305302601U, 10060288299563796713U,
16721032107124872262U, 1843161257003594899U, 1797766745982787050U,
390536904337743720U, 12106644034470055320U, 6371083594380682605U,
13036150688705644616U, 16739045801974399187U, 2401585797338929554U,
662183422668609739U, 2080942029017350301U, 9168970678409204496U,
7536929925042065149U, 12283898791789400255U, 11865981071280375610U,
17357275809210163231U, 3119971947938063298U, 4485410017532204164U,
9920502027531467780U, 12387657674122675340U, 15082506423611903223U,
6389097289230209530U, 13594575229040979271U, 15603462478660221875U,
4091990922018536136U, 16171254140317310531U, 3246788359678732845U,
8416718781492960135U, 2663865194348041573U, 8792844729951082315U,
14323769596549829169U, 1314999724015465477U, 3871503297060326582U,
9336137766612814354U, 12719109659353970748U, 6910773892072509259U,
11319805843442719264U, 425843746242816491U, 13201156133527311243U,
3407470517736513010U, 13397865681284145257U, 9505466498198367443U,
17968300338506116505U, 3614988282403063179U, 1384172312237648867U,
6015853531948011657U, 2024018753292845220U, 7404349130949546986U,
8173894174921337195U, 13583046464337282040U, 15246070772845607696U,
11459592115475048197U, 4759218179245013414U, 18408555040628554536U,
17262884048198642147U, 193827356580909706U, 6008648054008200887U,
1800648937158711358U, 479884830791397264U, 14876429754533315208U, 720547793981076U
};

const long flip[N] = {0, 256, 128, 384, 64, 320, 192, 448, 32, 288, 160, 416, 96, 352, 224,
480, 16, 272, 144, 400, 80, 336, 208, 464, 48, 304, 176, 432, 112, 368,
240, 496, 8, 264, 136, 392, 72, 328, 200, 456, 40, 296, 168, 424, 104,
360, 232, 488, 24, 280, 152, 408, 88, 344, 216, 472, 56, 312, 184, 440,
120, 376, 248, 504, 4, 260, 132, 388, 68, 324, 196, 452, 36, 292, 164,
420, 100, 356, 228, 484, 20, 276, 148, 404, 84, 340, 212, 468, 52, 308,
180, 436, 116, 372, 244, 500, 12, 268, 140, 396, 76, 332, 204, 460, 44,
300, 172, 428, 108, 364, 236, 492, 28, 284, 156, 412, 92, 348, 220, 476,
60, 316, 188, 444, 124, 380, 252, 508, 2, 258, 130, 386, 66, 322, 194,
450, 34, 290, 162, 418, 98, 354, 226, 482, 18, 274, 146, 402, 82, 338,
210, 466, 50, 306, 178, 434, 114, 370, 242, 498, 10, 266, 138, 394, 74,
330, 202, 458, 42, 298, 170, 426, 106, 362, 234, 490, 26, 282, 154, 410,
90, 346, 218, 474, 58, 314, 186, 442, 122, 378, 250, 506, 6, 262, 134,
390, 70, 326, 198, 454, 38, 294, 166, 422, 102, 358, 230, 486, 22, 278,
150, 406, 86, 342, 214, 470, 54, 310, 182, 438, 118, 374, 246, 502, 14,
270, 142, 398, 78, 334, 206, 462, 46, 302, 174, 430, 110, 366, 238, 494,
30, 286, 158, 414, 94, 350, 222, 478, 62, 318, 190, 446, 126, 382, 254,
510, 1, 257, 129, 385, 65, 321, 193, 449, 33, 289, 161, 417, 97, 353,
225, 481, 17, 273, 145, 401, 81, 337, 209, 465, 49, 305, 177, 433, 113,
369, 241, 497, 9, 265, 137, 393, 73, 329, 201, 457, 41, 297, 169, 425,
105, 361, 233, 489, 25, 281, 153, 409, 89, 345, 217, 473, 57, 313, 185,
441, 121, 377, 249, 505, 5, 261, 133, 389, 69, 325, 197, 453, 37, 293,
165, 421, 101, 357, 229, 485, 21, 277, 149, 405, 85, 341, 213, 469, 53,
309, 181, 437, 117, 373, 245, 501, 13, 269, 141, 397, 77, 333, 205, 461,
45, 301, 173, 429, 109, 365, 237, 493, 29, 285, 157, 413, 93, 349, 221,
477, 61, 317, 189, 445, 125, 381, 253, 509, 3, 259, 131, 387, 67, 323,
195, 451, 35, 291, 163, 419, 99, 355, 227, 483, 19, 275, 147, 403, 83,
339, 211, 467, 51, 307, 179, 435, 115, 371, 243, 499, 11, 267, 139, 395,
75, 331, 203, 459, 43, 299, 171, 427, 107, 363, 235, 491, 27, 283, 155,
411, 91, 347, 219, 475, 59, 315, 187, 443, 123, 379, 251, 507, 7, 263,
135, 391, 71, 327, 199, 455, 39, 295, 167, 423, 103, 359, 231, 487, 23,
279, 151, 407, 87, 343, 215, 471, 55, 311, 183, 439, 119, 375, 247, 503,
15, 271, 143, 399, 79, 335, 207, 463, 47, 303, 175, 431, 111, 367, 239,
495, 31, 287, 159, 415, 95, 351, 223, 479, 63, 319, 191, 447, 127, 383,
255, 511};


#endif

#endif // ifndef
